/*
 * Copyright 1993 by Ove Kalkan, Cremlingen, Germany
 *
 * Permission to use, copy, modify, distribute and sell this software and it's
 * documentation for any purpose is hereby granted without fee, rpovided that
 * the above copyright notice and this permission appear in supporting
 * documentation, and that the name of Ove Kalkan not to be used in
 * advertising or publicity pertaining to distributiopn of the software without
 * specific, written prior permission. Ove Kalkan makes no representations
 * about the suitability of this software for any purpose. It is provided
 * as is without express or implied warranty.
 *
 * OVE KALKAN DISPLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABLILITY AND FITNESS, IN NO
 * EVENT SHALL OVE KALKAN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * $Header: filename,v 1.0 yyyy/mm/dd hh:mm:ss loginname Exp $
 */


#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <dirent.h>

#include <grp.h>
#include <pwd.h>

#include <time.h>

#ifdef	hpux
#include <symlink.h>
#endif

#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Shell.h>

#include <X11/Xaw/Form.h>
#include <X11/Xaw/Simple.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Toggle.h>

#include "struct.h"
#include "config.h"

/*
 * Global variables
 */
extern	Widget	toplevel;		/* Das Toplevel widget fuer die Cursor */

extern	Cursor	def_cursor;
extern	Cursor	file_cursor;

Widget	dialog = NULL;			/* Das Dialogwidget */
Widget	info_shell = NULL;		/* Die Info-Shell */
Widget	text_1, text_2,
	text_3, text_4;				/* Zum Verschieben braucht man 2 Text,
					   sonst nur einen */
Widget	tog[9];				/* 9 Toggles */
char	link_target[255];

extern	Pixmap	Icon_Warning_PM;
extern	Pixmap	Icon_Info_PM;
extern	Pixmap	Icon_Help_PM;
extern	Pixmap	Icon_Filter_PM;

Pixmap	icon_pixmap;

extern	GC		line_gc;
extern	Boolean		FILE_CHANGED;
extern	Boolean		DIR_CHANGED;
extern	Folder_Glyph	*folder;
extern	Folder_Glyph	*selc_fo;
extern	Dimension	selc_f;
extern	Dir_Glyph	*selc_g;
extern	WS_Glyph	workspace;

char			*oldfile;
extern	xfm_struct	defaults;
extern	Suffix_Glyph	*filetypes;
extern	Dimension	max_filetypes;

/*
 * Function Prototypes
 */

void	init_setup     (Dir_Glyph *dir, File_Glyph *file);
void	init_quit      (void);
void	init_root      (void);
void	init_goto      (void);
void	hide_dialog    (void);
void	hide_idialog   (void);
void	exec_setup     (void);
void	WARNING        (char *text);
void	setSize        (void);
void	quit_all       (void);
Widget	makeDialog     (Dimension text, 
			char *label1, char *label2, 
			char *text1, char *text2, 
			char *title, Boolean cancel, Boolean OVERRIDE);

XtActionProc	refresh_dialog_icon    (Widget w, XEvent *e, 
					String *s, Cardinal *c);
extern	XtActionProc	refresh_files  (Widget w, XExposeEvent *e, 
					String *s, Cardinal *c);

void	init_info      (void);

void	apply_cb       (void);
void	reset_cb       (void);
void	getFilter (Folder_Glyph *folder);
void	apply_filter (Widget w, XtPointer c, XtPointer s);
#ifndef COHERENT
void	call_link_cb   (void);
extern	void	init_link(char *from, char *to);
#endif

extern	void	saveWorkspace (WS_Glyph *ws);
extern	Boolean	MULTI;



/*********************************************************
 * name:	hide_dialog
 * description:	loescht das Dialog-Fenster
 * input:	none
 * output:	none
 * date:	0.5.93
 *********************************************************/
void	hide_dialog (void)
{
	XUngrabPointer(XtDisplay(toplevel),CurrentTime);
	XtDestroyWidget (dialog);
	dialog = NULL;
}

/*********************************************************
 * name:	hide_dialog
 * description:	loescht das Dialog-Fenster
 * input:	none
 * output:	none
 * date:	0.5.93
 *********************************************************/
void	hide_idialog (void)
{
	if (oldfile)
		free(oldfile);
	XtDestroyWidget (info_shell);
	info_shell = NULL;
}


/*********************************************************
 * name:	setSize
 * description:	richtiges Setzen der Min-Max-Grenzen
 * input:	none
 * output:	none
 * date:	0.5.93
 *********************************************************/
void	setSize (void)
{
	Arg	args[3];
	Dimension	w,h;

	/*
	 * Max und Minsize setzen
	 */
	XtSetArg(args[0],XtNheight,&h);
	XtSetArg(args[1],XtNwidth,&w);
	XtGetValues(dialog,args,2);

	XtSetArg(args[2],XtNminWidth,w);
	XtSetArg(args[1],XtNmaxHeight,h);
	XtSetArg(args[0],XtNminHeight,h);
	XtSetValues(dialog,args,3);
}


/*********************************************************
 * name:	WARNING
 * description:	erzeugt ein Hinweisfenster
 * input:	char *text
 * output:	none
 * date:	10.6.93
 *********************************************************/
void	WARNING (char *text)
{
	Widget	but;

	/*
	 * Das Dialogfenster erzeugen
	 */
	icon_pixmap = Icon_Warning_PM;
	but = makeDialog(0,NULL,NULL,NULL,NULL,text,FALSE,TRUE);

	XtAddCallback(but,XtNcallback,(XtCallbackProc) hide_dialog,NULL);

	/*
	 * Zum Schluss die Dialogshell managen
	 */
	XtManageChild (dialog);
/*	XGrabPointer(XtDisplay(toplevel),XtWindow(toplevel), True,
			ButtonReleaseMask | ButtonPressMask,
			GrabModeAsync, GrabModeAsync, XtWindow(dialog), None,
			CurrentTime);
*/}

/*********************************************************
 * name:	init_goto
 * description:	erzeugt den requester fuer das goto
 * input:	none
 * output:	none
 * author:	Ove Kalkan
 * date:	5.8.1993
 *********************************************************/
void	init_goto (void)
{
	Widget	but;

	icon_pixmap = Icon_Help_PM;
	but = makeDialog(1,"Path:",NULL,NULL,NULL,"Show Directory - (NAY)",TRUE,FALSE);
	XtManageChild (dialog);
}

/*********************************************************
 * name:	init_root
 * description:	erzeugt den requester fuer das change root
 * input:	none
 * output:	none
 * author:	Ove Kalkan
 * date:	5.8.1993
 *********************************************************/
void	init_root (void)
{
	Widget	but;

	icon_pixmap = Icon_Help_PM;
	but = makeDialog(1,"Path:",NULL,NULL,NULL,"Change Root of Directorytree - (NAY)",TRUE,FALSE);
	XtManageChild (dialog);
}



/*********************************************************
 * name:	init_quit
 * description:	Fragen ob Programm beenden oder nicht
 * input:	none
 * output:	none
 * date:	10.6.93
 *********************************************************/
void	init_quit (void)
{
	Widget	but;

	/*
	 * Das Dialogfenster erzeugen
	 */
	icon_pixmap = Icon_Warning_PM;
	but = makeDialog(0,NULL,NULL,NULL,NULL,"Termintate Xfilemanager?",TRUE,TRUE);

	XtAddCallback(but,XtNcallback,(XtCallbackProc) quit_all,NULL);

	/*
	 * Zum Schluss die Dialogshell managen
	 */
	XtManageChild (dialog);
/*	XGrabPointer(XtDisplay(toplevel),XtWindow(toplevel), True,
			ButtonReleaseMask | ButtonPressMask,
			GrabModeAsync, GrabModeAsync, XtWindow(dialog), None,
			CurrentTime);
*/
}




/*
 * name:	makeDialog
 * description:	erzeugt ein Dialogfenster
 * input:	Dimension text	Anzahl der Textfenster
 *		char	*label1		- Label fuer Textfeld 1
 *		char	*label2		- Label fuer Textfeld 2
 *		char	*text1		- Text der in Textfeld 1 stehen soll
 *		char	*text2		- Text der in Textfeld 2 stehen soll
 *		char	*title		- Ueberschrift des Dialogs
 *		Boolean	cancel		- CancelButton erzeugen
 * output:	den Acceptbuttton zum setzen des Callbacks
 */
Widget	makeDialog(Dimension text, char *label1, char *label2, char *text1,
		   char *text2, char *title, Boolean cancel, Boolean OVERRIDE)
{
	Arg	args[11];
	Widget	form;
	Widget	icon;
	Widget	label;
	Widget	fl, tl;
	Widget	but, obut;
	int	w;

	/*
	 * Falls noch ein Dialog-Fenster geoeffnet ist, dann loeschen
	 */
	if (dialog)
		XtDestroyWidget(dialog);

	/*
	 * Das DialogWidget erzeugen
	 */
	if (OVERRIDE)
		dialog = XtVaCreatePopupShell ("dialog_shell", 
						overrideShellWidgetClass,
						toplevel, 
						XtNallowShellResize, TRUE,
						NULL);
	else
		dialog = XtVaCreatePopupShell ("dialog_shell",
						topLevelShellWidgetClass,
						toplevel, 
						XtNallowShellResize, TRUE,
						NULL);

	/*
	 * Die Form zum Anordnen der Widgets darauf erzeugen
	 */
	form = XtCreateManagedWidget ("dialog_form", formWidgetClass, dialog,
					args, 0);

	/*
	 * Den Simple fuer das Icon erstellen
	 */
	XtSetArg(args[0],XtNtop, XtChainTop);
	XtSetArg(args[1],XtNbottom, XtChainTop);
	XtSetArg(args[2],XtNright, XtChainLeft);
	XtSetArg(args[3],XtNleft, XtChainLeft);
	XtSetArg (args[6],XtNwidth, 32);
	XtSetArg (args[5],XtNheight, 32);
	XtSetArg (args[4],XtNborderWidth, 0);
	icon = XtCreateManagedWidget ("dialog_icon", simpleWidgetClass, form,
					args, 7);
	XtOverrideTranslations(icon,XtParseTranslationTable("<Expose>: refresh-dialog-icon()"));

	/*
	 * den Label darauf erzeugen, der sagt, das es sich hierbei um
	 * eine Verschiebe-Aktion handelt
	 */
	XtSetArg(args[5],XtNlabel, title);
	XtSetArg(args[6],XtNfromHoriz, icon);
	label = XtCreateManagedWidget ("dialog_title", labelWidgetClass, form,
					args, 7);
	tl = icon;
	if (strchr(title,'\n'))
		tl = label;

	if (text > 0) {
		/*
		 * Die Breite des Widgets fuer eine schoene Ausrichtung des
		 * Widgets ermitteln
		 */
		w = strlen(label1);
		if (text > 1 && strlen(label2) > w)
			w = strlen(label2);
		w *= 10;

		/*
		 * Nun die From Sachen (Label & Text)
		 */
		XtSetArg(args[5],XtNlabel, label1);
		XtSetArg(args[6],XtNfromVert, icon);
		XtSetArg(args[7],XtNwidth, w);
		XtSetArg(args[8],XtNjustify,XtJustifyLeft);
		fl = XtCreateManagedWidget ("dialog_from", labelWidgetClass, form,
						args, 9);

		XtSetArg(args[2],XtNright, XtChainRight);
		XtSetArg(args[4],XtNresizable,TRUE);
		XtSetArg(args[5],XtNfromHoriz, fl);
		XtSetArg(args[7],XtNeditType, XawtextEdit);
		XtSetArg(args[8],XtNresize, XawtextResizeWidth);
		XtSetArg(args[9],XtNstring, text1);
		XtSetArg(args[10], XtNwidth, 150);
		text_1 = XtCreateManagedWidget("dialog_text1", asciiTextWidgetClass,
						form, args,11);
		XtOverrideTranslations(text_1, XtParseTranslationTable(
						"<Key>Return: no-op()"));
		if (MULTI) {
			XtVaSetValues(text_1,
					XtNeditType,XawtextRead,
					XtNdisplayCaret,FALSE,
					XtNcursor, def_cursor,
					XtNheight, 70,
					XtNscrollVertical, XawtextScrollWhenNeeded,
					NULL);
		}
		tl = text_1;
	}

	if (text > 1) {
		/*
		 * Nun die To Sachen (Label & Text)
		 */
		XtSetArg(args[2],XtNright, XtChainLeft);
		XtSetArg(args[4],XtNborderWidth, 0);
		XtSetArg(args[5],XtNlabel, label2);
		XtSetArg(args[6],XtNfromVert, tl);
		XtSetArg(args[7],XtNwidth, w);
		XtSetArg(args[8],XtNjustify,XtJustifyLeft);
		fl = XtCreateManagedWidget ("dialog_from", labelWidgetClass, form,
						args, 9);

		XtSetArg(args[2],XtNright, XtChainRight);
		XtSetArg(args[4],XtNresizable,TRUE);
		XtSetArg(args[5],XtNfromHoriz, fl);
		XtSetArg(args[7],XtNeditType, XawtextEdit);
		XtSetArg(args[8],XtNresize, XawtextResizeWidth);
		XtSetArg(args[9],XtNstring, text2);
		text_2 = XtCreateManagedWidget("dialog_text2", asciiTextWidgetClass,
						form, args,11);
		XtOverrideTranslations(text_2, XtParseTranslationTable(
						"<Key>Return: no-op()"));
		tl = fl;
	}

	/*
	 * Zum Schluss noch die 2 Buttons
	 */
	but = NULL;
	XtSetArg(args[2],XtNright, XtChainLeft);
	XtSetArg(args[5],XtNfromVert, tl);
	XtSetArg(args[6],XtNvertDistance, 10);

	/*
	 * wenn gewuenscht, den Cancel Button
	 */
	if (cancel) {
		XtSetArg(args[4],XtNlabel, " Cancel ");
		but = XtCreateManagedWidget ("dialog_cancel", commandWidgetClass, form,
						args, 7);
		XtAddCallback(but,XtNcallback, (XtCallbackProc) hide_dialog, NULL);
	}

	/*
	 * Immer den OK-Button
	 */
	XtSetArg(args[4],XtNlabel, "   Ok   ");
	XtSetArg(args[7],XtNfromHoriz, but);
	obut = XtCreateManagedWidget ("dialog_accept", commandWidgetClass, form,
					args, 8);

	/*
	 * Hier die Position der Maus feststellen und das Fenster so plazieren,
	 * das der Mauszeiger genau ueber dem Cancel-Button ist
	 */
	{
		Window	a,b;
		int	ax,ay,bx,by;
		unsigned int	m;

		XQueryPointer(XtDisplay(toplevel),XtWindow(toplevel),
				&a,&b,&ax,&ay,&bx,&by,&m);
		XtMoveWidget (dialog,(0 > ax - 30 ? 0 : ax - 30),
				     (0 > ay - 60 ? 0 : ay - 60));
	}
	return(obut);
}


/*********************************************************
 * name:	refresh_dialog_icon
 * description:	Wenn das Icon im Dialog-Fenster uebermalt
 *		wurde, dann wird es hiermit wieder neu
 *		gezeichnet.
 *		In icon_pixmap ist der Zeiger auf die
 *		Pixmap zu finden.
 * input:	none
 * output:	none
 * date:	14.6.93
 *********************************************************/
XtActionProc	refresh_dialog_icon(Widget w, XEvent *e, String *s, Cardinal *c)
{
	/*
	 * einfach nur die mit icon_pixmap spezifizierte Pixmap in das Simplewidget
	 * kopieren. Die gewuenschte Pixmap wird diesem Zeiger vorher zugewiesen
	 */
	XCopyArea (XtDisplay(w),icon_pixmap, XtWindow(w), line_gc,
			0,0,32,32,0,0);
}


/*********************************************************
 * name:	quit_all
 * description:	Programmende, nachdem Quit bestaetigt wurde
 * input:	none
 * output:	none
 * date:	14.6.93
 *********************************************************/
void	quit_all (void)
{
	if (defaults.save_ws_on_exit)
		saveWorkspace(&workspace);
	exit(0);
}



/*********************************************************
 * name:	init_info
 * description:	erzeugt den file_info-dialog
 * input:	none
 * output:	none
 * author:	Ove Kalkan
 * date:	18.6.93
 *********************************************************/
void	init_info(void)
{
	Widget	form, icon,f;
	Widget	label, w, bw[4];
	extern	char *getString(void);
	extern	char *getPath(Dir_Glyph *dir);
	char	*file;
	Arg	args[11];
	struct	stat	buf;
	char	dummy[256];
	uid_t	uid = getuid();
	gid_t	gid = getgid();
	int	b = 0;
	struct	passwd *pw;
	struct	group *gr;
	WidgetClass	class;


	/*
	 * erst alten Dialog zerstoeren
	 */
	if (info_shell)
		XtDestroyWidget(info_shell);

	oldfile = getString();		/* Filenamen zum verifizieren sichern */

	/*
	 * Das DialogWidget erzeugen
	 */
	XtSetArg(args[0],XtNallowShellResize,TRUE);
	XtSetArg(args[1],XtNtransientFor, toplevel);
	info_shell = XtCreatePopupShell ("dialog_shell", topLevelShellWidgetClass,
					toplevel, args, 2);

	/*
	 * Die Form zum Anordnen der Widgets darauf erzeugen
	 */
	form = XtCreateManagedWidget ("dialog_form", formWidgetClass, info_shell,
					args, 0);
	/*
	 * Den Simple fuer das Icon erstellen
	 */
	icon_pixmap = Icon_Info_PM;
	XtSetArg(args[0],XtNtop, XtChainTop);
	XtSetArg(args[1],XtNbottom, XtChainTop);
	XtSetArg(args[2],XtNright, XtChainLeft);
	XtSetArg(args[3],XtNleft, XtChainLeft);
	XtSetArg (args[4],XtNwidth, 32);
	XtSetArg (args[5],XtNheight, 32);
	XtSetArg (args[6],XtNborderWidth, 0);
	icon = XtCreateManagedWidget ("dialog_icon", simpleWidgetClass, form,
					args, 7);
	XtOverrideTranslations(icon,XtParseTranslationTable("<Expose>: refresh-dialog-icon()"));

	/*
	 * den Label darauf erzeugen, der sagt, das es sich hierbei um
	 * eine Verschiebe-Aktion handelt
	 */
	XtSetArg(args[4],XtNborderWidth, 0);
	XtSetArg(args[5],XtNfromHoriz, icon);
	XtSetArg(args[6],XtNlabel,"File-Information");
	label = XtCreateManagedWidget ("info_title", labelWidgetClass, form,
					args, 7);

	/*
	 * Den Status des Files fuer die uebrigen Boxen holen
	 */
	file = getString();
	lstat (file,&buf);

	/*
	 * Die Box fuer den Filenamen
	 */
	XtSetArg(args[4],XtNborderWidth, 0);
	XtSetArg(args[5],XtNwidth, 150);
	XtSetArg(args[6],XtNjustify, XtJustifyRight);
	XtSetArg(args[7],XtNlabel,"Filename :");
	XtSetArg(args[8],XtNfromVert, icon);
	label = XtCreateManagedWidget ("info_name", labelWidgetClass, form,
					args, 9);

	free (file);
	if (selc_g)
		file = selc_g->name;
	else
		file = selc_fo->file[selc_f]->name;

	XtSetArg(args[2],XtNright, XtChainRight);
	XtSetArg(args[4],XtNfromHoriz, label);
	XtSetArg(args[5],XtNfromVert, icon);
	if ((uid == buf.st_uid || uid == 0) && strcmp(file,"..")) {	/* Filename veraenderbar fuer Owner
						   und Root */
		XtSetArg(args[6],XtNeditType, XawtextEdit);
		XtSetArg(args[7],XtNstring, file);
		XtSetArg(args[8],XtNresize, XawtextResizeWidth);
		XtSetArg(args[9],XtNwidth, 13*strlen(file));
		text_1 = XtCreateManagedWidget("name_text", asciiTextWidgetClass,
						form, args,10);
		XtOverrideTranslations(text_1, XtParseTranslationTable(
						"<Key>Return: no-op()"));
	}
	else {
		XtSetArg(args[6],XtNlabel, file);
		XtSetArg(args[7],XtNborderWidth, 0);
		XtSetArg(args[8],XtNjustify, XtJustifyLeft);
		text_1 = XtCreateManagedWidget("name_text", labelWidgetClass,
						form, args,9);
	}

#ifndef COHERENT
	/* Es handelt sich um einen Link, also noch das Target ausgeben */
	text_4 = NULL;
	if ((selc_g && (selc_g->flags & DIR_LINK)) || (selc_fo && selc_f < 16000 &&
	     (selc_fo->file[selc_f]->prog_type == FILE_LINK ||
	      selc_fo->file[selc_f]->prog_type == FILE_LDIR))) {
		int	length = 0;
		int	flag = 0;

		XtSetArg(args[2],XtNright,XtChainLeft);
		XtSetArg(args[3],XtNleft,XtChainLeft);
		XtSetArg(args[4],XtNborderWidth,0);
		XtSetArg(args[5],XtNlabel,"Link to :");
		XtSetArg(args[7],XtNfromVert, text_1);
		XtSetArg(args[6],XtNjustify, XtJustifyRight);
		XtSetArg(args[8],XtNwidth,150);
		label = XtCreateManagedWidget("link_info",labelWidgetClass,form,args,9);

		file = getString();
		if (oldfile)
			length = readlink(oldfile,link_target,255);
		if (length < 0) {
			flag = 1;
			sprintf(link_target,"Link failure");
		}
		else
			link_target[length] = '\0';

		XtSetArg(args[2],XtNright,XtChainLeft);
		XtSetArg(args[4],XtNfromHoriz, label);
		XtSetArg(args[5],XtNfromVert, text_1);
		if ((uid == buf.st_uid || uid == 0) && !flag) {
			XtSetArg(args[6],XtNlabel, link_target);
			text_4 = XtCreateManagedWidget("link_text", commandWidgetClass,
							form, args,7);
			XtAddCallback(text_4,XtNcallback,(XtCallbackProc) call_link_cb,NULL);
		}
		else {
			XtSetArg(args[6],XtNlabel, link_target);
			XtSetArg(args[7],XtNborderWidth, 0);
			XtSetArg(args[8],XtNjustify, XtJustifyLeft);
			text_4 = XtCreateManagedWidget("name_text", labelWidgetClass,
							form, args,9);
		}
	}
	else 
#endif	/* Coherent Unix kann keine Links */
	{
		XtSetArg(args[2],XtNright, XtChainLeft);
		XtSetArg(args[3],XtNleft, XtChainLeft);
		XtSetArg(args[4],XtNborderWidth, 0);
		XtSetArg(args[5],XtNwidth, 150);
		XtSetArg(args[6],XtNjustify, XtJustifyRight);
		XtSetArg(args[7],XtNlabel,"Filetype :");
		XtSetArg(args[8],XtNfromVert, text_1);
		label = XtCreateManagedWidget ("info_name", labelWidgetClass, form,
					args, 9);
		/*
		 * Kurzbeschreibung
		 */
		XtSetArg(args[5],XtNfromHoriz,label);
		XtSetArg(args[8],XtNfromVert,text_1);
		if (selc_g)
			XtSetArg(args[7],XtNlabel,"Directory");
		else if (selc_fo && selc_f < 16000) {
			File_Glyph	*ff = selc_fo->file[selc_f];

			if (ff->prog_type < max_filetypes) {
				XtSetArg(args[7],XtNlabel,
					 filetypes[ff->prog_type].description);
			}
			else {
				switch (ff->prog_type) {
				case FILE_EXEC:
					XtSetArg(args[7],XtNlabel,"Executable");
					break;
				case FILE_DIR:
					XtSetArg(args[7],XtNlabel,"Directory");
					break;
				case FILE_ROOT:
					XtSetArg(args[7],XtNlabel,"Parent Directory");
					break;
				default:
					XtSetArg(args[7],XtNlabel,"Unknown File");
				}
			}
		}
		else
			XtSetArg(args[7],XtNlabel,"Unknown File");
		label = XtCreateManagedWidget ("info_name", labelWidgetClass, form,
						args, 9);
	}
	w = label;

	/*
	 * Die Box fuer den Filepath
	 */
	XtSetArg(args[2],XtNright, XtChainLeft);
	XtSetArg(args[3],XtNleft, XtChainLeft);
	XtSetArg(args[4],XtNborderWidth,0);
	XtSetArg(args[6],XtNjustify, XtJustifyRight);
	XtSetArg(args[5],XtNwidth, 150);
	XtSetArg(args[7],XtNlabel,"Location :");
	XtSetArg(args[8],XtNfromVert, w);
	label = XtCreateManagedWidget ("info_name", labelWidgetClass, form,
					args, 9);

	if (selc_g) {
		if (selc_g->parent)
			file = getPath(selc_g->parent);
		else
			file = "";
	}
	else
		file = getPath(selc_fo->dir);

	XtSetArg(args[6],XtNfromHoriz, label);
	XtSetArg(args[5],XtNfromVert, w);
	XtSetArg(args[7],XtNlabel, file);
	w = XtCreateManagedWidget("location_text", labelWidgetClass,
					form, args,8);
	free(file);

	/*
	 * Die Box fuer den Filegroesse
	 */
	XtSetArg(args[2],XtNright, XtChainLeft);
	XtSetArg(args[4],XtNborderWidth, 0);
	XtSetArg(args[5],XtNwidth, 150);
	XtSetArg(args[6],XtNjustify, XtJustifyRight);
	XtSetArg(args[7],XtNlabel,"Size :");
	XtSetArg(args[8],XtNfromVert, w);
	label = XtCreateManagedWidget ("info_name", labelWidgetClass, form,
					args, 9);

	sprintf(dummy,"%d Bytes",buf.st_size);
	XtSetArg(args[6],XtNfromHoriz, label);
	XtSetArg(args[5],XtNfromVert, w);
	XtSetArg(args[7],XtNlabel, dummy);
	w = XtCreateManagedWidget("location_text", labelWidgetClass,
					form, args,8);

	/*
	 * Die Box fuer den Owner
	 */
	XtSetArg(args[2],XtNright, XtChainLeft);
	XtSetArg(args[4],XtNborderWidth, 0);
	XtSetArg(args[5],XtNwidth, 150);
	XtSetArg(args[6],XtNjustify, XtJustifyRight);
	XtSetArg(args[7],XtNlabel,"Owner :");
	XtSetArg(args[8],XtNfromVert, w);
	label = XtCreateManagedWidget ("info_owner", labelWidgetClass, form,
					args, 9);

	/*
	 * Username holen
	 */
	pw = getpwuid(buf.st_uid);
	if (pw)
		sprintf(dummy,"%s",pw->pw_name);
	else
		sprintf(dummy,"Unknown User");

	XtSetArg(args[4],XtNfromVert, w);
	XtSetArg(args[5],XtNfromHoriz, label);
	if (uid == buf.st_uid || uid == 0) {	/* Filename veraenderbar fuer Owner
						   und Root */
		XtSetArg(args[2],XtNright, XtChainRight);
		XtSetArg(args[6],XtNeditType, XawtextEdit);
		XtSetArg(args[7],XtNstring,  dummy);
		XtSetArg(args[8],XtNwidth, 13*strlen(dummy));
		XtSetArg(args[9],XtNresize, XawtextResizeWidth);
		text_2 = XtCreateManagedWidget("name_text", asciiTextWidgetClass,
						form, args,10);
		XtOverrideTranslations(text_2, XtParseTranslationTable(
						"<Key>Return: no-op()"));
	}
	else {
		XtSetArg(args[6],XtNlabel, dummy);
		XtSetArg(args[7],XtNborderWidth, 0);
		XtSetArg(args[8],XtNjustify, XtJustifyLeft);
		text_2 = XtCreateManagedWidget("name_text", labelWidgetClass,
						form, args,9);
	}
	w = label;

	/*
	 * Die Box fuer den Groups
	 */
	XtSetArg(args[2],XtNright, XtChainLeft);
	XtSetArg(args[4],XtNborderWidth, 0);
	XtSetArg(args[5],XtNwidth, 150);
	XtSetArg(args[6],XtNjustify, XtJustifyRight);
	XtSetArg(args[7],XtNlabel,"Group :");
	XtSetArg(args[8],XtNfromVert, w);
	label = XtCreateManagedWidget ("info_group", labelWidgetClass, form,
					args, 9);

	/*
	 * Groupname holen
	 */
	gr = getgrgid(buf.st_gid);
	if (gr)
		sprintf(dummy,"%s",gr->gr_name);
	else
		sprintf(dummy,"Unknown Group");
	XtSetArg(args[4],XtNfromVert, w);
	XtSetArg(args[5],XtNfromHoriz, label);
	if (uid == buf.st_uid || uid == 0) {	/* Filename veraenderbar fuer Owner
						   und Root */
		XtSetArg(args[2],XtNright, XtChainRight);
		XtSetArg(args[6],XtNeditType, XawtextEdit);
		XtSetArg(args[7],XtNstring,  dummy);
		XtSetArg(args[8],XtNwidth, 13*strlen(dummy));
		XtSetArg(args[9],XtNresize, XawtextResizeWidth);
		text_3 = XtCreateManagedWidget("name_text", asciiTextWidgetClass,
						form, args,10);
		XtOverrideTranslations(text_3, XtParseTranslationTable(
						"<Key>Return: no-op()"));
	}
	else {
		XtSetArg(args[6],XtNlabel, dummy);
		XtSetArg(args[7],XtNborderWidth, 0);
		XtSetArg(args[8],XtNjustify, XtJustifyLeft);
		text_3 = XtCreateManagedWidget("name_text", labelWidgetClass,
						form, args,9);
	}
	w = label;

	/*
	 * Die Box fuer die Createion Time
	 */
	XtSetArg(args[2],XtNright, XtChainLeft);
	XtSetArg(args[4],XtNborderWidth, 0);
	XtSetArg(args[5],XtNwidth, 150);
	XtSetArg(args[6],XtNjustify, XtJustifyRight);
	XtSetArg(args[7],XtNlabel,"Creation Time :");
	XtSetArg(args[8],XtNfromVert, w);
/*	label = XtCreateManagedWidget ("info_group", labelWidgetClass, form,
					args, 9);

	{
		struct	tm	*tt;

		tt = localtime (&buf.st_ctime);
		sprintf(dummy,"%d.%d.%d",tt->tm_mday,tt->tm_mon+1,1900+tt->tm_year);
	}
	XtSetArg(args[8],XtNfromVert,w);
	XtSetArg(args[7],XtNlabel,dummy);
	XtSetArg(args[5],XtNfromHoriz,label);
	label = XtCreateManagedWidget ("creat_time", labelWidgetClass, form,
					args,9);
	w = label;
*/
	/*
	 * Die Box fuer die Modification Time
	 */
	XtSetArg(args[5],XtNwidth, 150);
	XtSetArg(args[7],XtNlabel,"Last Modification :");
	XtSetArg(args[8],XtNfromVert, w);
	label = XtCreateManagedWidget ("info_group", labelWidgetClass, form,
					args, 9);

	{
		struct	tm	*tt;

		tt = localtime (&buf.st_mtime);
		if (tt)
			sprintf(dummy,"%d.%d.%d",tt->tm_mday,tt->tm_mon+1,1900+tt->tm_year);
		else
			sprintf(dummy,"No Time");
	}
	XtSetArg(args[8],XtNfromVert,w);
	XtSetArg(args[7],XtNlabel,dummy);
	XtSetArg(args[5],XtNfromHoriz,label);
	label = XtCreateManagedWidget ("creat_time", labelWidgetClass, form,
					args,9);
	w = label;

	/*
	 * Nun die Dialogbox fuer die Rechte
	 */
	XtSetArg(args[2],XtNright, XtChainLeft);
	XtSetArg(args[4],XtNborderWidth, 0);
	XtSetArg(args[5],XtNwidth, 80);
	XtSetArg(args[6],XtNjustify, XtJustifyLeft);
	XtSetArg(args[7],XtNlabel,"");
	XtSetArg(args[8],XtNfromVert, w);
	label = XtCreateManagedWidget ("info_empty", labelWidgetClass, form,
					args, 9);	
	XtSetArg(args[6],XtNjustify, XtJustifyCenter);
	XtSetArg(args[5],XtNwidth, 70);
	XtSetArg(args[7],XtNlabel,"Read");
	XtSetArg(args[9],XtNfromHoriz,label);
	bw[1] = XtCreateManagedWidget ("info_ruser", labelWidgetClass, form,
					args, 10);	
	XtSetArg(args[7],XtNlabel,"Write");
	XtSetArg(args[9],XtNfromHoriz,bw[1]);
	bw[2] = XtCreateManagedWidget ("info_wuser", labelWidgetClass, form,
					args, 10);	
	XtSetArg(args[7],XtNlabel,"Execute");
	XtSetArg(args[9],XtNfromHoriz,bw[2]);
	bw[3] = XtCreateManagedWidget ("info_xuser", labelWidgetClass, form,
					args, 10);

	/*
	 * Rechte des Owners darstellen
	 */
	w = label;
	XtSetArg(args[6],XtNjustify, XtJustifyRight);
	XtSetArg(args[5],XtNwidth, 80);
	XtSetArg(args[8],XtNfromVert, w);
	XtSetArg(args[7],XtNlabel,"Owner :");		
	label = XtCreateManagedWidget ("info_empty", labelWidgetClass, form,
					args, 9);

	if (uid == buf.st_uid || uid == 0) {
		class = toggleWidgetClass;
		XtSetArg(args[4],XtNborderWidth, 1);
		XtSetArg(args[6],XtNjustify, XtJustifyCenter);
		XtSetArg(args[5],XtNwidth, 70);
		XtSetArg(args[9],XtNfromHoriz,label);
		XtSetArg(args[7],XtNlabel,"  ");
		XtSetArg(args[10],XtNstate, (S_IRUSR&buf.st_mode ? TRUE : FALSE));
		tog[0] = XtCreateManagedWidget ("toggle", class, form,
					args, 11);
		XtSetArg(args[7],XtNlabel,"  ");
		XtSetArg(args[9],XtNfromHoriz,tog[0]);
		XtSetArg(args[10],XtNstate, (S_IWUSR&buf.st_mode ? TRUE : FALSE));
		tog[1] = XtCreateManagedWidget ("toggle", class, form,
						args, 11);
		XtSetArg(args[7],XtNlabel,"  ");
		XtSetArg(args[9],XtNfromHoriz,tog[1]);
		XtSetArg(args[10],XtNstate, (S_IXUSR&buf.st_mode ? TRUE : FALSE));
		tog[2] = XtCreateManagedWidget ("toggle", class, form,
						args, 11);
	}
	else {
		class = labelWidgetClass;
		XtSetArg(args[7],XtNlabel, (S_IRUSR&buf.st_mode ? "Yes" : "No"));
		XtSetArg(args[4],XtNborderWidth, 1);
		XtSetArg(args[6],XtNjustify, XtJustifyCenter);
		XtSetArg(args[5],XtNwidth, 70);
		XtSetArg(args[9],XtNfromHoriz,label);
		tog[0] = XtCreateManagedWidget ("toggler", class, form,
						args, 10);	
		XtSetArg(args[7],XtNlabel, (S_IWUSR&buf.st_mode ? "Yes" : "No"));
		XtSetArg(args[9],XtNfromHoriz,tog[0]);
		tog[1] = XtCreateManagedWidget ("toggler", class, form,
						args, 10);
		XtSetArg(args[7],XtNlabel, (S_IXUSR&buf.st_mode ? "Yes" : "No"));
		XtSetArg(args[9],XtNfromHoriz,tog[1]);
		tog[2] = XtCreateManagedWidget ("toggler", class, form,
						args, 10);
	}

	/*
	 * Rechte der Group darstellen
	 */
	w = label;
	XtSetArg(args[4],XtNborderWidth, 0);
	XtSetArg(args[6],XtNjustify, XtJustifyRight);
	XtSetArg(args[5],XtNwidth, 80);
	XtSetArg(args[8],XtNfromVert, w);
	XtSetArg(args[7],XtNlabel,"Group :");		
	label = XtCreateManagedWidget ("info_empty", labelWidgetClass, form,
					args, 9);

	if (uid == buf.st_uid || uid == 0) {
		class = toggleWidgetClass;
		XtSetArg(args[4],XtNborderWidth, 1);
		XtSetArg(args[6],XtNjustify, XtJustifyCenter);
		XtSetArg(args[5],XtNwidth, 70);
		XtSetArg(args[9],XtNfromHoriz,label);
		XtSetArg(args[7],XtNlabel,"  ");
		XtSetArg(args[10],XtNstate, (S_IRGRP&buf.st_mode ? TRUE : FALSE));
		tog[3] = XtCreateManagedWidget ("toggle", class, form,
					args, 11);
		XtSetArg(args[7],XtNlabel,"  ");
		XtSetArg(args[9],XtNfromHoriz,tog[3]);
		XtSetArg(args[10],XtNstate, (S_IWGRP&buf.st_mode ? TRUE : FALSE));
		tog[4] = XtCreateManagedWidget ("toggle", class, form,
						args, 11);
		XtSetArg(args[7],XtNlabel,"  ");
		XtSetArg(args[9],XtNfromHoriz,tog[4]);
		XtSetArg(args[10],XtNstate, (S_IXGRP&buf.st_mode ? TRUE : FALSE));
		tog[5] = XtCreateManagedWidget ("toggle", class, form,
						args, 11);
	}
	else {
		class = labelWidgetClass;
		XtSetArg(args[7],XtNlabel, (S_IRGRP&buf.st_mode ? "Yes" : "No"));
		XtSetArg(args[4],XtNborderWidth, 1);
		XtSetArg(args[6],XtNjustify, XtJustifyCenter);
		XtSetArg(args[5],XtNwidth, 70);
		XtSetArg(args[9],XtNfromHoriz,label);
		tog[3] = XtCreateManagedWidget ("tog_ruser", class, form,
						args, 10);	
		XtSetArg(args[7],XtNlabel, (S_IWGRP&buf.st_mode ? "Yes" : "No"));
		XtSetArg(args[9],XtNfromHoriz,tog[3]);
		tog[4] = XtCreateManagedWidget ("tog_wuser", class, form,
						args, 10);
		XtSetArg(args[7],XtNlabel, (S_IXGRP&buf.st_mode ? "Yes" : "No"));
		XtSetArg(args[9],XtNfromHoriz,tog[4]);
		tog[5] = XtCreateManagedWidget ("tog_xuser", class, form,
						args, 10);
	}

	/*
	 * Rechte der Others darstellen
	 */
	w = label;
	XtSetArg(args[4],XtNborderWidth, 0);
	XtSetArg(args[6],XtNjustify, XtJustifyRight);
	XtSetArg(args[5],XtNwidth, 80);
	XtSetArg(args[8],XtNfromVert, w);
	XtSetArg(args[7],XtNlabel,"Other :");		
	label = XtCreateManagedWidget ("info_empty", labelWidgetClass, form,
					args, 9);

	if (uid == buf.st_uid || uid == 0) {
		class = toggleWidgetClass;
		XtSetArg(args[4],XtNborderWidth, 1);
		XtSetArg(args[6],XtNjustify, XtJustifyCenter);
		XtSetArg(args[5],XtNwidth, 70);
		XtSetArg(args[9],XtNfromHoriz,label);
		XtSetArg(args[7],XtNlabel,"  ");
		XtSetArg(args[10],XtNstate, (S_IROTH&buf.st_mode ? TRUE : FALSE));
		tog[6] = XtCreateManagedWidget ("toggle", class, form,
					args, 11);
		XtSetArg(args[7],XtNlabel,"  ");
		XtSetArg(args[9],XtNfromHoriz,tog[6]);
		XtSetArg(args[10],XtNstate, (S_IWOTH&buf.st_mode ? TRUE : FALSE));
		tog[7] = XtCreateManagedWidget ("toggle", class, form,
						args, 11);
		XtSetArg(args[7],XtNlabel,"  ");
		XtSetArg(args[9],XtNfromHoriz,tog[7]);
		XtSetArg(args[10],XtNstate, (S_IXOTH&buf.st_mode ? TRUE : FALSE));
		tog[8] = XtCreateManagedWidget ("toggle", class, form,
						args, 11);
	}
	else {
		class = labelWidgetClass;
		XtSetArg(args[7],XtNlabel, (S_IROTH&buf.st_mode ? "Yes" : "No"));
		XtSetArg(args[4],XtNborderWidth, 1);
		XtSetArg(args[6],XtNjustify, XtJustifyCenter);
		XtSetArg(args[5],XtNwidth, 70);
		XtSetArg(args[9],XtNfromHoriz,label);
		tog[6] = XtCreateManagedWidget ("tog_ruser", class, form,
						args, 10);	
		XtSetArg(args[7],XtNlabel, (S_IWOTH&buf.st_mode ? "Yes" : "No"));
		XtSetArg(args[9],XtNfromHoriz,tog[6]);
		tog[7] = XtCreateManagedWidget ("tog_wuser", class, form,
						args, 10);
		XtSetArg(args[7],XtNlabel, (S_IXOTH&buf.st_mode ? "Yes" : "No"));
		XtSetArg(args[9],XtNfromHoriz,tog[7]);
		tog[8] = XtCreateManagedWidget ("tog_xuser", class, form,
						args, 10);
	}

	/*
	 * Zum Schluss noch eine Form fuer die Buttons erzeugen
	 */
	XtSetArg(args[0],XtNtop, XawChainBottom);
	XtSetArg(args[1],XtNbottom, XawChainBottom);
	XtSetArg(args[2],XtNleft, XawChainLeft);
	XtSetArg(args[3],XtNright, XawChainRight);
	XtSetArg(args[4],XtNborderWidth,0);
	XtSetArg(args[5],XtNfromVert, label);
	XtSetArg(args[6],XtNvertDistance, 20);
	f = XtCreateManagedWidget("info_cform",formWidgetClass,form,args,7);

	/*
	 * In der neuen Form die Commandbuttons erzeugen
	 */
	XtSetArg(args[0],XtNleft, XawChainLeft);
	XtSetArg(args[1],XtNright, XawChainLeft);
	XtSetArg(args[2],XtNlabel, "  Hide  ");
	w = XtCreateManagedWidget("hide_info",commandWidgetClass,f,args,3);
	XtAddCallback(w,XtNcallback,(XtCallbackProc) hide_idialog,NULL);

	{
		int	a = (uid == buf.st_uid ? 0 : 1);

		XtSetArg(args[0],XtNleft, XawChainLeft);
		XtSetArg(args[1],XtNright, XawChainLeft);
		XtSetArg(args[2],XtNlabel, " Reset ");
		XtSetArg(args[3],XtNhorizDistance,160);
		XtSetArg(args[4],XtNfromHoriz, w);
		XtSetArg(args[5],XtNsensitive,FALSE);
		w = XtCreateManagedWidget("hide_info",commandWidgetClass,f,args,5+a);
		XtAddCallback(w,XtNcallback,(XtCallbackProc) reset_cb,NULL);

		XtSetArg(args[0],XtNleft, XawChainLeft);
		XtSetArg(args[1],XtNright, XawChainLeft);
		XtSetArg(args[2],XtNlabel, " Apply ");
		XtSetArg(args[3],XtNfromHoriz, w);
		XtSetArg(args[4],XtNsensitive,FALSE);
		w = XtCreateManagedWidget("apply_info",commandWidgetClass,f,args,4+a);
		XtAddCallback(w,XtNcallback,(XtCallbackProc) apply_cb,NULL);
	}

	XtManageChild(info_shell);
}



/*********************************************************
 * name:	apply_cb
 * description:	change modes, owners and names of files
 * input:	none
 * output:	none
 * author:	Ove Kalkan
 * date:	1.7.93
 *********************************************************/
void	apply_cb(void)
{
	Boolean		b;
	Arg		args[1];
	mode_t		mode = 0;
	struct	stat	buf;
	char		*of, *nf;

	/*
	 * Bisherige Rechte des Files holen und nur die Benutzerrechte,
	 * keine speziellen flags loeschen
	 */
	lstat(oldfile,&buf);
	mode = buf.st_mode & ~(S_IRUSR | S_IWUSR | S_IXUSR |
			       S_IRGRP | S_IWGRP | S_IXGRP |
			       S_IROTH | S_IWOTH | S_IXOTH);

	/*
	 * Benutzerrechte holen und setzen
	 */
	XtSetArg(args[0],XtNstate,&b);
	XtGetValues(tog[0],args,1);
	mode |= (b ? S_IRUSR : 0);
	XtGetValues(tog[1],args,1);
	mode |= (b ? S_IWUSR : 0);
	XtGetValues(tog[2],args,1);
	mode |= (b ? S_IXUSR : 0);
	XtGetValues(tog[3],args,1);
	mode |= (b ? S_IRGRP : 0);
	XtGetValues(tog[4],args,1);
	mode |= (b ? S_IWGRP : 0);
	XtGetValues(tog[5],args,1);
	mode |= (b ? S_IXGRP : 0);
	XtGetValues(tog[6],args,1);
	mode |= (b ? S_IROTH : 0);
	XtGetValues(tog[7],args,1);
	mode |= (b ? S_IWOTH : 0);
	XtGetValues(tog[8],args,1);
	mode |= (b ? S_IXOTH : 0);

	if (mode != buf.st_mode)
		chmod (oldfile,mode);

	/*
	 * File umbenennen falls erforderlich
	 */
	XtSetArg(args[0],XtNstring,&nf);
	XtGetValues(text_1,args,1);

	of = strrchr(oldfile,'/') + 1;
	if (strcmp(of,nf)) {
		char 	*p;

		p = strrchr(oldfile,'/');
		if (p) {
			int	i;
			char	sa[1024];

			*p = '\0';
			sprintf(sa,"mv %s/%s %s/%s",oldfile,of,oldfile,nf);
			SYSTEM(sa);

			i = strlen(oldfile);
			*p = '/';
			/* Oldfile wird fuer das chown auf newfile gesetzt */
			if (!(oldfile = (char *) realloc((void *) oldfile,
							 i + strlen(nf) + 2)))
				FATAL_ERROR("apply_cb: realloc failed");
			strncpy(&oldfile[i+1],nf,strlen(nf));
			oldfile[i+1+strlen(nf)] = '\0';
		}
	}
	/*
	 * Group und Owner setzen - Owner zuletzt
	 */
	{
		char	*grp, *own;
		uid_t	uid;
		gid_t	gid;

		struct	passwd	*pws;
		struct	group	*grs;

		/*
		 * Namen der Group oder des Owners holen
		 */
		XtSetArg(args[0],XtNstring, &grp);
		XtGetValues(text_3,args,1);
		XtSetArg(args[0],XtNstring, &own);
		XtGetValues(text_2,args,1);
		/*
		 * Zu den Namen die UserId und die GroupId holen, falls moeglich
		 */

		pws = getpwnam (own);
		if (pws) {
			uid = pws->pw_uid;
		}
		else {
			WARNING("Change Owner: Not a valid user");
			return;
		}
		grs = getgrnam (grp);
		if (grs) {
			gid = grs->gr_gid;
		}
		else {
			WARNING("Change Group: Not a valid group");
			return;
		}
		chown(oldfile,uid,gid);
	}
}


/*********************************************************
 * name:	reset_cb
 * description:	reset modes, owners and names of files
 * input:	none
 * output:	none
 * author:	Ove Kalkan
 * date:	1.7.93
 *********************************************************/
void	reset_cb(void)
{
	struct	stat	buf;
	Arg		args[1];
	struct	passwd	*pw;
	struct	group	*gr;
	char		*b;

	/*
	 * Status des urspruenglich angewaehlten Files wieder herstellen
	 */
	lstat(oldfile,&buf);

	/*
	 * Filenamen ruecksetzen
	 */
	b = strrchr(oldfile,'/');
	XtSetArg(args[0],XtNstring,b+1);
	XtSetValues(text_1,args,1);

	/*
	 * Owner und Groups ruecksetzen
	 */
	pw = getpwuid(buf.st_uid);
	XtSetArg(args[0],XtNstring,pw->pw_name);
	XtSetValues(text_2,args,1);

	gr = getgrgid(buf.st_gid);
	XtSetArg(args[0],XtNstring,gr->gr_name);
	XtSetValues(text_3,args,1);

	/*
	 * Rightflags ruecksetzen
	 */
	XtSetArg(args[0],XtNstate, (S_IRUSR&buf.st_mode ? TRUE : FALSE));
	XtSetValues(tog[0],args,1);
	XtSetArg(args[0],XtNstate, (S_IWUSR&buf.st_mode ? TRUE : FALSE));
	XtSetValues(tog[1],args,1);
	XtSetArg(args[0],XtNstate, (S_IXUSR&buf.st_mode ? TRUE : FALSE));
	XtSetValues(tog[2],args,1);
	XtSetArg(args[0],XtNstate, (S_IRGRP&buf.st_mode ? TRUE : FALSE));
	XtSetValues(tog[3],args,1);
	XtSetArg(args[0],XtNstate, (S_IWGRP&buf.st_mode ? TRUE : FALSE));
	XtSetValues(tog[4],args,1);
	XtSetArg(args[0],XtNstate, (S_IXGRP&buf.st_mode ? TRUE : FALSE));
	XtSetValues(tog[5],args,1);
	XtSetArg(args[0],XtNstate, (S_IROTH&buf.st_mode ? TRUE : FALSE));
	XtSetValues(tog[6],args,1);
	XtSetArg(args[0],XtNstate, (S_IWOTH&buf.st_mode ? TRUE : FALSE));
	XtSetValues(tog[7],args,1);
	XtSetArg(args[0],XtNstate, (S_IXOTH&buf.st_mode ? TRUE : FALSE));
	XtSetValues(tog[8],args,1);

}



/*********************************************************
 * name:	getFilter
 * description:	Den Filterstring fuer einen Folder berechnen
 * input:	Folder_Glyph *folder;
 * output:	none
 * author:	Ove Kalkan
 * date:	2.7.1993
 *********************************************************/
void	getFilter (Folder_Glyph *folder)
{
	Widget	but;

	/*
	 * Das Dialogfenster erzeugen
	 */
	icon_pixmap = Icon_Filter_PM;
	but = makeDialog(1,"Filter :",NULL,NULL,NULL,"Filefilter",TRUE,FALSE);

	XtAddCallback(but,XtNcallback,(XtCallbackProc) apply_filter,(XtPointer) folder);

	/*
	 * Zum Schluss die Dialogshell managen
	 */
	XtManageChild (dialog);

	/*
	 * Max und Minsize setzen
	 */
	setSize();
}



/*********************************************************
 * name:	apply_filter
 * description:	Den Filterstring fuer einen Folder setzen
 *		und refreshen
 * input:	none
 * output:	none
 * author:	Ove Kalkan
 * date:	2.7.1993
 *********************************************************/
void	apply_filter (Widget w, XtPointer c, XtPointer s)
{
	char		*a;
	Folder_Glyph	*f = (Folder_Glyph *) c;
	Arg		args[1];

	XtSetArg(args[0],XtNstring,&a);
	XtGetValues(text_1,args,1);
	if (f->filter)
		free(f->filter);
	if (strlen(a))
		f->filter = a;
	else
		f->filter = NULL;
	XtDestroyWidget(dialog);
	dialog = NULL;
	if (selc_fo == f) {
		selc_fo = f;
		selc_f = 16000;
	}
	fillFolder(f);
	FILE_CHANGED = TRUE;
	refresh_files(f->window,NULL,NULL,NULL);
}


#ifndef COHERENT
/*********************************************************
 * name:	call_link_cb
 * description:	aufrufen von init_link
 * input:	none
 * output:	none
 * author:	Ove Kalkan
 * date:	Friday, 13th.Aug.1993
 *********************************************************/
void	call_link_cb(void)
{
	init_link(oldfile,link_target);
}
#endif
