/*
  Xinvest is copyright 1995 by Mark Buser.

  Permission is hereby granted to copy
  and freely distribute copies of this
  program for non-commercial purposes
  without fee, provided that this notice
  appears in all copies.
  
  All redistributions must be in their
  entirety as originally distributed.
  Feel free to modify Xinvest, but
  modified versions may not be distributed
  without prior consent of the author.
  
  This software is provided 'as-is'
  without any express or implied warranty.
  In no event will the author be held
  liable for any damages resulting from
  the use of this software.

  $Revision: 2.14 $ $Date: 1997/02/11 01:20:17 $
*/

#include <math.h>
#ifdef STRFMON
#include <monetary.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>

#include "account.h"
#include "calc.h"
#include "color.h"
#include "drawing.h"
#include "nav.h"
#include "rate.h"
#include "status.h"
#include "trans.h"
#include "xutil.h"

extern Widget Gaincalctext;

/* Set calculator display to val, compute returns and display if calc is true */
void displayCalc ( double val, int calc )
{
  char calc_string[20];
  
  sprintf( calc_string, "$%f", val );
  XmTextSetString ( Gaincalctext, calc_string);

  /* Compute return, as a result, redrawDrawing */
  if (calc)
    compute();
}

/* ARGSUSED */
void calc_push (  Widget w, XtPointer client_data, XtPointer call_data )
{
  char *newdisplay, *curdisplay;
  int  state = 0; /* 0 is new, 1 is append, 2 is found "." */
                  /* 3 is freeze */
  static int freeze = True;
  char *dot;
  char next[8];
 
  curdisplay = XmTextGetString ( Gaincalctext );
  newdisplay = XtMalloc ( strlen(curdisplay) +1);

  dot = strchr (curdisplay, '.');
  strcpy (newdisplay, curdisplay); 

  if (strlen(newdisplay) > 1)   
     state = 1;
  else 
     state = 0;

  if (dot) state = 2;

  if (freeze) state = 0;

  XtFree ( curdisplay );

  if ( strcmp( newdisplay, "$0") == 0) { /* if just leading zero, remove it */
    strcpy( newdisplay, "$");
    state = 0;
  }

  switch ( (int)client_data ) {
    case 0: strcpy(next, "7"); break;
    case 1: strcpy(next, "8"); break;
    case 2: strcpy(next, "9"); break;
    case 3: strcpy(next, "4"); break;
    case 4: strcpy(next, "5"); break;
    case 5: strcpy(next, "6"); break;
    case 6: strcpy(next, "1"); break;
    case 7: strcpy(next, "2"); break;
    case 8: strcpy(next, "3"); break;
    case 9: if (state == 0) 
              strcpy(next, "0."); 
            else if (state == 1) 
              strcpy(next, ".");
            else if (state == 2)
              strcpy(next, "");
            break;
    case 10: strcpy(next, "0");                     
             break;
    case 11: state = 0; 
             strcpy( next, "0");
             break;
    case 12: state = 3; 
             compute(); 
             break;
  }


  if ( state == 0) {                   /* new, trash old chars */
    strcpy( newdisplay, "$");
    strcat( newdisplay, next);
  } else if (state == 1 || state == 2){
    if (strlen( newdisplay) > strlen("12345.67") ) {
      strcpy( newdisplay, " ERROR! ");
    } else
      strcat ( newdisplay, next );
  }

  if (state != 3) {
    freeze = False;
    XmTextSetString ( Gaincalctext, newdisplay ); 
  } else
    freeze = True;

  XtFree (newdisplay);
  
}

void compute ()
{
  char *nav;
  char *sans_dollar;
  double price = 0.0;

  int account = activeAccount();
  NAV *navp;

  getAccount ( account, ACCOUNT_NAV, &navp);

  /* Get NAV from screen */
  nav = XmTextGetString ( Gaincalctext );
  sans_dollar = strchr( nav, '$');
  if (sans_dollar)
    price = atof(sans_dollar +1);      /* +1, skip '$' */
  XtFree ( nav );

  setNav ( navp, NAV_VALUE, &price );

  drawDrawingArea( );
}


/* Set NAV date/value to that of last transaction and redisplay */
/* ARGSUSED */
void lastPush (Widget w, XtPointer client_data, XtPointer call_data)
{
  int account = activeAccount();
  void *trans;
  int numTrans;
  NAV *navp;
  double trans_nav;

  getAccount ( account, ACCOUNT_TRANS, &trans);
  getAccount ( account, ACCOUNT_NUM_TRANS, &numTrans);

  getAccount ( account, ACCOUNT_NAV, &navp);
  trans_nav = GetTransNav( trans, numTrans);

  setNav (navp, NAV_VALUE, &trans_nav);
  setNav (navp, NAV_DATE, GetTransDate( trans, numTrans));

  displayAccount (account);
}


static int multiAccount = False;

/* Set return mode, account or portfolio */
/* ARGSUSED */
void calcMode (Widget w, XtPointer client_data, XtPointer call_data)
{
  XmToggleButtonCallbackStruct *state =
         (XmToggleButtonCallbackStruct *) call_data;
  int save = multiAccount;

  if ( state->set )
    multiAccount = True;
  else
    multiAccount = False;

  if (multiAccount != save)
    drawDrawingArea();
}


#define INFO_FIELDS 9  /* Title plus info */
#define INFO_LINES  6
#define CENTER 0
#define LEFT 1
#define RIGHT 2

void draw_return ( Widget Drawing, Pixmap Drawpixmap, GC gc )
{

  Dimension height, width;
  Pixel foreground, background;

  extern XFontStruct *large;
  extern XFontStruct *small;

  /* user title from transaction file */
  int  account = activeAccount();
  NAV  *navp;      
  NAV  *lastDate = (NAV *)NULL;
  int  navChanged = False;

  int  i;
  char string[80];
  int noballs;
  int pos, start, end;
  int textWidth;
  int textHeight;
  int x, y;

  Pixel    color;
  Pixel    box_shadow, bot_shadow;

  double   port_value = 0.0, port_cost = 0.0, port_debit = 0.0, port_fee = 0.0;
  double   port_distrib = 0.0;

  /* Did we already get a too wide/high error */
  int    errors = 0;

  XtVaGetValues ( Drawing, XmNheight, &height,
                           XmNwidth,  &width,
                           XmNforeground, &foreground,
                           XmNbackground, &background,
                  NULL);

  if ( isColorVisual ( XtDisplay(Drawing) ) ) {
    XmGetColors ( XtScreen (Drawing), GetColormap(), background,
                  NULL, NULL, &bot_shadow, NULL );
    box_shadow = bot_shadow;
  } else {
    bot_shadow = background;
    box_shadow = GetColor (BLACK);
  }

  /* Set colors */
  XSetForeground ( XtDisplay (Drawing), gc, foreground );
  XSetBackground ( XtDisplay (Drawing), gc, background );
  XSetFont ( XtDisplay (Drawing), gc, large->fid );

  if (!multiAccount) {
    getAccount( account, ACCOUNT_NAV, &navp );      
    lastDate = navp;
    navChanged = isNavChanged( account, navp );
    port_value = account_value( account, navp );
    port_cost = account_cost( account, navp );
    port_debit = account_withdrawal( account, navp );
    port_fee = account_fee( account, navp );
    port_distrib = account_distrib (account, navp);
  } else {
    for (account = 1; account <= numAccounts(); account++) {
      getAccount( account, ACCOUNT_NAV, &navp );      
      if (lastDate == (NAV *)NULL || navToLDate(lastDate) < navToLDate (navp) )
        lastDate = navp;
      if ( isNavChanged( account, navp ) )
        navChanged = True;
      port_value += account_value( account, navp );
      port_cost += account_cost( account, navp );
      port_debit += account_withdrawal( account, navp );
      port_fee += account_fee( account, navp );
      port_distrib += account_distrib (account, navp);
    }
  }

  for (i=0; i < INFO_FIELDS; i++) {

    /* Generate strings */
    switch (i) {

       case 0:  /* title and date */
                {
                  char strdate[40];
                  struct tm date;
                  int m, d, y;

                  getNav (lastDate, NAV_MONTH, &m);
                  getNav (lastDate, NAV_DAY, &d);
                  getNav (lastDate, NAV_YEAR, &y);
                  sprintf ( strdate, "%d/%d/%d", m, d, y);
                  strptime ( strdate, "%m/%d/%Y", &date);
                  strftime ( strdate, 40, "%x", &date );

                  if (!multiAccount) {
                    char *name = createAccountName (account);
                    sprintf ( string, " %.20s - %.20s ", name, strdate );
                    XtFree (name);
                  } else
                    sprintf ( string, " Open Account(s) - %.20s ", strdate );

                  noballs = True;
                  pos = CENTER;
                  break;
                }

       case 1:  /* price */
                if (!multiAccount) {
                  float p;
                  getNav(navp, NAV_VALUE, &p); 
#ifdef STRFMON
                  strfmon ( string, 80, " Price: %.2n ", (double)p);
#else
                  sprintf ( string, " Price: $%.2f ", p );
#endif
                } else
                  strcpy ( string, " Price: NA " );

                noballs = False;
                pos = LEFT;
                break;

       case 2:  /* shares */
                if (!multiAccount) {
#ifdef STRFMON
                  strfmon ( string, 80, " Shares: %!.3n ", 
                            account_shares( account, navp ));
#else
                  sprintf ( string, " Shares: %.3f ", 
                            account_shares( account, navp ));
#endif
                } else 
                  strcpy ( string, " Shares: NA " );

                noballs = False;
                pos = RIGHT;
                break;

       case 3:  /* value */
#ifdef STRFMON
                strfmon ( string, 80, " Value: %.2n ", port_value );
#else
                sprintf ( string, " Value: $%.2f ", port_value );
#endif
                noballs = False;
                pos = LEFT;
                break;

       case 4:  /* cost */
#ifdef STRFMON
                strfmon ( string, 80," Cost: %.2n ", port_cost - port_debit );
#else
                sprintf ( string, " Cost: $%.2f ", port_cost - port_debit );
#endif

                noballs = False;
                pos = RIGHT;
                break;

       case 5:  /* fees */
                if (port_fee > 0.0)
                  port_fee = -port_fee;
#ifdef STRFMON
                strfmon ( string, 80," Fee: %.2n ",
                          (port_fee > 0.0)?-port_fee:port_fee );
#else
                sprintf ( string, " Fee: $%.2f ",
                          (port_fee > 0.0)?-port_fee:port_fee );
#endif

                noballs = False;
                pos = LEFT;
                break;

       case 6:  /* distributions */
#ifdef STRFMON
                strfmon ( string, 80," Debit: %.2n ",
                          (port_debit > 0.0)?-port_debit:port_debit );
#else
                sprintf ( string, " Debit: $%.2f ",
                          (port_debit > 0.0)?-port_debit:port_debit );
#endif

                noballs = False;
                pos = RIGHT;
                break;

       case 7:  /* IRR */
                { 
		  double rate;
                  rateClrAccount( );

                  if (!multiAccount) {
                    getAccount( account, ACCOUNT_NAV, &navp );      
                    rateAddAccount ( account, navp );
                  } else {
                    for (account = 1; account <= numAccounts(); account++) {
                      getAccount( account, ACCOUNT_NAV, &navp );      
                      rateAddAccount ( account, navp );
                    }
                  }
                  rate = rate_IRR ( port_value + port_distrib );
#ifdef STRFMON
                  strfmon ( string, 78, " Annualized Return: %!.2n", rate);
                  strcat ( string, "% ");  /*  bug in Solaris strfmon */
#else
                  sprintf( string, " Annualized Return: %.2f%% ", rate);
#endif

                  noballs = False;
                  pos = CENTER;
                  break;
                }

       case 8:  /* TR */
                { 
                  double rate;
                  if (port_cost == (double)0.0)
                    rate = 0.0;
                  else
                    rate = ((port_value + port_debit - port_cost) / port_cost)
                           * 100.0;
#ifdef STRFMON
                  strfmon ( string, 78, " Total Return: %!.2n", rate);
                  strcat ( string, "% ");
#else
                  sprintf ( string, " Total Return: %.2f%% ", rate );
#endif
                  noballs = False;
                  pos = CENTER;
                  break;
                }
    }

    /* Draw the strings */

    textWidth = XTextWidth ( large, string, strlen (string) );
    textHeight = large->ascent + large->descent;

    if (pos == CENTER) {
      x = (int)(width - textWidth)/2;
      start = 0;
      end = width;
    } else if (pos == LEFT) {
      x = (int)(width/2 - textWidth)/2;
      start = 0;
      end = width/2;
    } else {
      x = (int)(width/2 - textWidth)/2 + width/2;
      start = width/2;
      end = width;
    }

    if (i == 0)
      y = (int)( height -  small->ascent - small->descent - 
                 (textHeight + large->descent)*INFO_LINES) / 2;

    /* textHeight looks weird in an x dimension, but its used to space
    ** the ball wrt the font actually used.
    */
    /* have text and ball plus text < width and y < height */
    if ( strcmp ( string, "" ) != 0 &&
         bound_ok ( x+textWidth+textHeight, end, &errors, i ) &&
         bound_ok ( start, x-textHeight, &errors, i  ) &&
         bound_ok ( 0, y, &errors, INFO_FIELDS +i ) &&
         bound_ok ( y, height, &errors, INFO_FIELDS +i ) ) {

      XSetForeground ( XtDisplay (Drawing), gc, box_shadow );
      XFillRectangle ( XtDisplay (Drawing), Drawpixmap, gc, 
                       (noballs)? (x+2):(x -textHeight +2), y +2, 
                       (noballs)? (textWidth):(textWidth+textHeight), textHeight
                     );
      XSetForeground ( XtDisplay (Drawing), gc, background );
      XFillRectangle ( XtDisplay (Drawing), Drawpixmap, gc, 
                       (noballs)? (x):(x -textHeight), y, 
                       (noballs)? (textWidth):(textWidth+textHeight), textHeight
                     );

      XSetForeground ( XtDisplay (Drawing), gc, bot_shadow );
      XDrawString ( XtDisplay (Drawing), Drawpixmap, gc,
                    x +1, y + large->ascent +1,
                    string, strlen ( string) );
      XSetForeground ( XtDisplay (Drawing), gc, foreground );
      XDrawString ( XtDisplay (Drawing), Drawpixmap, gc,
                    x, y + large->ascent,
                    string, strlen ( string) );

      if (noballs == False) {
        int neg;
        int symbol_border;
        int symbol_size;

        if ( strpbrk ( string, "-(") ) {
          color = GetColor ( RED );
          neg = True;
        } else {
          color = GetColor ( GREEN );
          neg = False;
        }

        if ( i < 5) { /* Price, Shares, Value, Cost */
          symbol_border = (int)(0.2*textHeight);
          symbol_size = (int)(0.6*textHeight);

          XSetForeground ( XtDisplay (Drawing), gc, box_shadow );
          XFillArc ( XtDisplay (Drawing), Drawpixmap, gc,
                     x - symbol_size +1, y + symbol_border +1,
                     symbol_size, symbol_size, 0, 360*64 );

          XSetForeground ( XtDisplay (Drawing), gc, color );
          XFillArc ( XtDisplay (Drawing), Drawpixmap, gc,
                   x - symbol_size, y + symbol_border,
                   symbol_size, symbol_size, 0, 360*64 );

        } else {  /*  IRR, TR */

          int one_third, one_half, two_third;
          XPoint point[7];

          symbol_border = (int)(0.1*textHeight);
          symbol_size = (int)(0.8*textHeight);
          one_third = symbol_size/3;
          two_third = one_third *2;
          one_half = symbol_size/2;

          /******************
               ^6
             /   \
          0--1   4--5
             |___|
             2   3
          *******************/
          point[0].x = x - symbol_size +1;
          point[0].y = (neg)?(y + symbol_border + one_third +1):(y + symbol_border + two_third +1);
          point[1].x = one_third;
          point[1].y = 0;
          point[2].x = 0;
          point[2].y = (neg)?-one_third:one_third;
          point[3].x = one_third;
          point[3].y = 0;
          point[4].x = 0;
          point[4].y = -point[2].y;
          point[5].x = point[1].x;
          point[5].y = 0;
          point[6].x = -one_half;
          point[6].y = (neg)?two_third:-two_third;

          XSetForeground ( XtDisplay (Drawing), gc, box_shadow );
          XFillPolygon ( XtDisplay(Drawing), Drawpixmap, gc,
                         point, 7, Convex, CoordModePrevious );

          point[0].x--;
          point[0].y--;
          XSetForeground ( XtDisplay (Drawing), gc, color );
          XFillPolygon ( XtDisplay(Drawing), Drawpixmap, gc,
                         point, 7, Convex, CoordModePrevious );
        }
      }

      if (pos == CENTER || pos == RIGHT)
        y += textHeight + large->descent;
    }
  }

  strcpy ( string, "");

  /* Let them know they changed the NAV */
  if ( navChanged ) 
     strcat( string, "** Value reflects changed NAV. ");

  if ( strlen(string) ) {
    XSetFont ( XtDisplay (Drawing), gc, small->fid );
    textWidth = XTextWidth ( small, string, strlen ( string ) );
    textHeight = small->ascent + small->descent;
    x = (int)(width - textWidth)/2;

    if ( bound_ok ( x+textWidth, width, &errors, 2*INFO_FIELDS ) &&
         bound_ok ( 0, x, &errors, 2*INFO_FIELDS+1 ) ) {

      XSetForeground ( XtDisplay (Drawing), gc, box_shadow );
      XFillRectangle ( XtDisplay (Drawing), Drawpixmap, gc, 
                       x+2, y +2, textWidth, textHeight);
      XSetForeground ( XtDisplay (Drawing), gc, background );
      XFillRectangle ( XtDisplay (Drawing), Drawpixmap, gc, 
                       x, y, textWidth, textHeight);

      XSetForeground ( XtDisplay (Drawing), gc, bot_shadow );
      XDrawString ( XtDisplay (Drawing), Drawpixmap, gc,
                    x +1, y + small->ascent + 1, string, strlen ( string ) );
      XSetForeground ( XtDisplay (Drawing), gc, foreground );
      XDrawString ( XtDisplay (Drawing), Drawpixmap, gc,
                     x, y + small->ascent, string, strlen ( string ) );
    }
  }

  /* If anything went awry, pop-up a message */
  /* We can get very specific, since we know where it went wrong. */
  if ( errors && !suppressed() ) {
    if ( errors < (1 << INFO_FIELDS) ) 
       write_status ("Text too long, reduce \"large\" font size\n"
                     "or make window wider.  See \"help Fonts\".", 
                     WARN );
    else if ( errors >= (1 << 2*INFO_FIELDS) ) 
       write_status ("Text too long, reduce \"large\" font size\n"
                     "or make window wider.  See \"help Fonts\".", 
                     WARN );
    else
       write_status ("Graph area too small, increase vertical size.", 
                     WARN);
  }
}
