/*
  Xinvest is copyright 1995,1996 by Mark Buser.

  Permission is hereby granted to copy
  and freely distribute copies of this
  program for non-commercial purposes
  without fee, provided that this notice
  appears in all copies.
  
  All redistributions must be in their
  entirety as originally distributed.
  Feel free to modify Xinvest, but
  modified versions may not be distributed
  without prior consent of the author.
  
  This software is provided 'as-is'
  without any express or implied warranty.
  In no event will the author be held
  liable for any damages resulting from
  the use of this software.

  $Revision: 2.20 $ $Date: 1997/05/22 23:32:29 $
*/

#include <stdio.h>
#include <time.h>
#include <unistd.h>

#include <X11/Xlib.h>
#include <Xm/XmAll.h>

#ifdef EDITRES
#include <X11/Xmu/Editres.h>
#endif

#include "account.h"
#include "calc.h"
#include "calendar.h"
#include "color.h"
#include "drawing.h"
#include "edit.h"
#include "file.h"
#include "graph.h"
#include "nav.h"
#include "pixmap.h"
#include "rate.h"
#include "remote.h"
#include "session.h"
#include "status.h"
#include "util.h"
#include "xinvest.h"
#include "xutil.h"

/* Menu & button callbacks */
extern void fileCB ();
       void toolCB ();
extern void accountCB();
extern void statusCB();
extern void helpCB();

/* Event handlers */
extern void showDetailEH();


/* Globals */
static char version[] = "Xinvest 2.4";

Month months[] = {
  { "January" }, { "February" }, { "March" }, { "April" },
  { "May" }, { "June" }, { "July" }, { "August" }, 
  { "September" }, { "October" }, { "November" }, { "December" }
};


static char *digits[] = { "seven", "eight", "nine", 
                          "four",  "five",  "six", 
                          "one",   "two",   "three", 
                          "pnt",   "zero",  "clr", 
                                  "cmp"
                        };

static char *portoptmenu[] = { "Display", "Root", "Accounts", "Flatten" };

static char *calcfuncs[] =   { "Function", "Future Value (Lump)", 
                               "Future Value (Series)", "Periodic Payment", 
                               "Present Value", "Rate" };

       char *calcparams[] =  { "Present Value", "Future Value", "Rate", 
                        "Compounded", "Years", "Payment"};
static char *calccompnd[] =  { "", "Daily", "Weekly", "Monthly", "Quarterly", 
                        "Bi-annually", "Annually", "180/365", "360/365" };

static MENUITEMS file[] = {
 { "button_0",    XmVaPUSHBUTTON, fileCB },
 { "button_1",    XmVaPUSHBUTTON, fileCB },
 { "button_2",    XmVaPUSHBUTTON, fileCB },
 { "button_3",    XmVaPUSHBUTTON, fileCB },
 { "button_4",    XmVaPUSHBUTTON, fileCB },
 { "separator_0", XmVaSEPARATOR,  NULL },
 { "button_5",    XmVaPUSHBUTTON, fileCB }
};

static MENUITEMS edit[] = {
 { "button_0",    XmVaPUSHBUTTON, editCB },
 { "button_1",    XmVaPUSHBUTTON, editCB },
 { "button_2",    XmVaPUSHBUTTON, editCB }
};

static MENUITEMS tool[] = {
 { "button_0",    XmVaRADIOBUTTON, toolCB },
 { "button_1",    XmVaRADIOBUTTON, toolCB },
 { "button_2",    XmVaRADIOBUTTON, toolCB },
 { "button_3",    XmVaRADIOBUTTON, toolCB },
 { "button_4",    XmVaRADIOBUTTON, toolCB },
};

static MENUITEMS account[] = {
 { "button_0",    XmVaPUSHBUTTON, accountCB },
 { "button_1",    XmVaPUSHBUTTON, accountCB },
 { "separator_0", XmVaSEPARATOR,   NULL }
};

static MENUITEMS log[] = {
 { "button_0",    XmVaPUSHBUTTON, statusCB }
};

static MENUITEMS option[] = {
 { "button_0",    XmVaCHECKBUTTON, graph_option },
 { "button_1",    XmVaCHECKBUTTON, graph_option },
 { "button_2",    XmVaCHECKBUTTON, graph_option }
};

static MENUITEMS help[] = {
 { "button_0",    XmVaPUSHBUTTON, helpCB }
};

static BUTTONITEMS toolbar[] = {
  { "About", XmVaPUSHBUTTON, (Pixmap)NULL, (Pixmap)NULL, (Pixmap)NULL,
    (XtPointer)NULL, showDetailEH, toolCB, NULL 
  },
  { "Gain", XmVaPUSHBUTTON, (Pixmap)NULL, (Pixmap)NULL, (Pixmap)NULL,
    (XtPointer)NULL, showDetailEH, toolCB, NULL 
  },
  { "Graphbut",	XmVaPUSHBUTTON, (Pixmap)NULL, (Pixmap)NULL, (Pixmap)NULL,
    (XtPointer)NULL, showDetailEH, toolCB, NULL 
  },
  { "Calculate", XmVaPUSHBUTTON, (Pixmap)NULL, (Pixmap)NULL, (Pixmap)NULL,
    (XtPointer)NULL, showDetailEH, toolCB, NULL 
  },
  { "Portfolio", XmVaPUSHBUTTON, (Pixmap)NULL, (Pixmap)NULL, (Pixmap)NULL,
    (XtPointer)NULL, showDetailEH, toolCB, NULL 
  },
  { "Quit", XmVaPUSHBUTTON, (Pixmap)NULL, (Pixmap)NULL, (Pixmap)NULL,
    (XtPointer)NULL, showDetailEH, fileCB, NULL 
  }
};

static BUTTONITEMS gainbar[] = {
  { "Gaincalclast", XmVaPUSHBUTTON, (Pixmap)NULL, (Pixmap)NULL, (Pixmap)NULL,
    (XtPointer)NULL, showDetailEH, lastPush, NULL 
  },
  { "Gaincalcmode", XmVaTOGGLEBUTTON, (Pixmap)NULL, (Pixmap)NULL, (Pixmap)NULL,
    (XtPointer)NULL, showDetailEH, calcMode, NULL 
  }
};


XFontStruct *small, *large;

Widget Filemenu; 
Widget Editmenu; 
static Widget Toolmenu; 
Widget Optionmenu; 
Widget Accountmenu; 
Widget Transtext; 
Widget Translabel;
Widget Filecwdlabel;
Widget Listfile;
static Widget Infoframe;
static Widget Gainform;
Widget Gaincalrow; 
static Widget Gaindaterow; 
Widget Gainmonthoptmenu; 
Widget Gainyeartext; 
Widget Gaincalctext; 
static Widget Graphform; 
Widget Graphhbar; 
Widget Graphhline; 
Widget GraphconstToggle; 
Widget Graphscale; 
Widget Graphcform;  
static Widget Portform;
Widget Portpbut;
Widget Portmbut;
Widget Portcbut;
Widget Portabut;
Widget Portpercentlist;
Widget Portoptmenu;
Widget Portvaluelabel; 
static Widget Calcform;
Widget Calclabel;
Widget Calclabels[5];
Widget Calctext[3]; 
Widget Savebut;
Widget Cyclebut; 
Widget Gainbut; 
Widget Graphbut;
Widget Portbut;

/* 
** Forward Declarations 
*/

/* Portfolio callbacks */
extern void  portAddSubList();
extern void  portTraverse();

/* Calculator callbacks */
extern void  calcFuncSel();
extern void  calcTextSel();
extern void  calcCompSel();

void main (int argc, char **argv)
{

  XtAppContext app;

  /* application resources */
  AppData appdata;

  static String fallback_resources[] = {
#include "defaults.h"
         NULL,
  };

  XmFontContext   fontContext;

  /* Main Window Widgets */
  Widget Toplevel;
  Widget Mainform;
   Widget Menubar;
/*   Widget Filemenu; */
/*   Widget Editmenu; */
/*   Widget Toolmenu; */
/*   Widget Accountmenu; */
/*   Widget Optionmenu; */
   Widget Workform;
    Widget HPane;    /* manages left side of Workform */
     Widget VPane;   /* manages left side of Workform */
/*    Widget Translabel; */
/*    Widget Transtext; */
      Widget Graphframe;
        Widget Graph;   
     Widget Midform; /* manages center of Workform */
/*     Widget Infoframe; */
       Widget Fileframe;
/*     Widget Filecwdlabel; */
         Widget Fileform;
/*        Widget Listfile; */
/*        Widget Savebut; */
          Widget Newbut;
          Widget Openbut;
/*        Widget Cyclebut; */
       Widget Actframe;
         Widget Actform; /* The following share the same screen area */
/*        Widget Graphform; */
            Widget Graphpframe;
              Widget Graphprow;
            Widget Graphhframe;
              Widget Graphhrow;
            Widget Graphvframe;
              Widget Graphvform;
                Widget Graphvscroll;
/*              Widget Graphcform; */
                  Widget Graphvtext1;
                  Widget Graphvlabel; 
                  Widget Graphvtext2;
/*              Widget Graphscale; */
                Widget Graphslabel;
/*        Widget Portform; */
           Widget Portselrow;
            Widget Portframe;
             Widget Portbutform;
/*            Widget Portpbut; */
/*            Widget Portmbut; */
/*            Widget Portabut; */
/*            Widget Portcbut; */
           Widget Portscrform;
            Widget Portpercentlabel;
/*          Widget Portpercentlist; */
/*          Widget Portoptmenu; */
/*          Widget Portvaluelabel; */
/*        Widget Gainform; */
           Widget Gaincalframe;
/*          Widget Gaincalrow; */
/*           Widget Gaindaterow; */
/*            Widget Gainmonthoptmenu; */
              Widget GainyearUparrow; 
/*            Widget Gainyeartext; */
              Widget GainyearDnarrow; 
/*           Widget Months[].form */
           Widget Gainvalframe;
            Widget Gainvalrow;
/*           Widget Gaincalctext; */
             Widget Gaincalcrow;
           Widget Gainbutframe;
            Widget Gainbutform;
/*        Widget Calcform; */
           Widget Calcoptmenu;
           Widget Calcframe;
/*           Widget Calclabel; */
           Widget Calcrow;
/*	    Widget Calclabels[5]; */
/*	    Widget Calctext[3]; */
            Widget Calccmpdmenu;
     Widget Butform;

  /* Display */
  Display *dpy;

  /* Window */
  Window win;

  /* Color resources */
  Colormap cmap;
  GC       gc;

  /* X strings */
  XmString Temp[ XtNumber (calccompnd)];
  XmString strs[ XtNumber (months) ];

  /* array of resources */
  int n;
  Arg args[15];

  char  dir[BUFSIZ];
  int   month;

  int    i;  /* ubiquitous i */
  Widget w;  /* temporary widget handle */

  /* Initialize toolkit */
  XtSetLanguageProc (NULL, NULL, NULL);

#if XtSpecificationRelease > 5
  /* R6 session shell supports session management */
  Toplevel = XtVaOpenApplication( &app, "Xinvest",
                                  options, XtNumber(options), 
                                  &argc, argv, 
                                  fallback_resources, 
                                  sessionShellWidgetClass, NULL);
  /* These callbacks are required */
  XtAddCallback (Toplevel, XtNsaveCallback, sessSave, 0);
  XtAddCallback (Toplevel, XtNcancelCallback, sessResume, 0);
  XtAddCallback (Toplevel, XtNsaveCompleteCallback, sessResume, 0);
  XtAddCallback (Toplevel, XtNdieCallback, sessDie, 0);

#else
  /* Old style WM_SAVE_YOURSELF property method of save state notification */
  Toplevel = XtVaAppInitialize( &app, "Xinvest",
                                options, XtNumber(options), &argc, argv, 
                                fallback_resources, NULL);
  {
     Atom XaWmSaveYourself = XmInternAtom ( XtDisplay (Toplevel), 
		                            "WM_SAVE_YOURSELF", False);
     XmAddWMProtocols ( Toplevel, &XaWmSaveYourself, 1);
     XmAddWMProtocolCallback ( Toplevel, XaWmSaveYourself, sessSave, 0);
  }
#endif

  XmRepTypeInstallTearOffModelConverter();
  XtVaGetApplicationResources ( Toplevel, 
                                &appdata, 
                                resources, 
                                XtNumber( resources ),
                                NULL );

  /* If help, print syntax and exit */
  if (appdata.help) {
    syntax (argc, argv);
    exit(0);
  }

#ifdef EDITRES
  XtAddEventHandler( Toplevel, 0, True, _XEditResCheckMessages, NULL);
#endif

  XtAppAddActions ( app, actions, XtNumber(actions) );

  dpy = XtDisplay( Toplevel );
  win = RootWindowOfScreen (XtScreen(Toplevel));
  cmap = DefaultColormapOfScreen (XtScreen(Toplevel));
  gc = XCreateGC ( dpy, win, 0, NULL );

  /**********************/

  /* Get a color map */
  if (appdata.install)
    cmap = NewColormap (dpy, cmap );

  { /* Application resource colors get added to color list. */
    Pixel appcolors[8];

    appcolors[0] = appdata.color0; 
    appcolors[1] = appdata.color1; 
    appcolors[2] = appdata.color2; 
    appcolors[3] = appdata.color3; 
    appcolors[4] = appdata.color4; 
    appcolors[5] = appdata.color5; 
    appcolors[6] = appdata.color6; 
    appcolors[7] = appdata.color7; 

    cmap = InitColor ( Toplevel, cmap, appcolors );
    if ( cmap == (Colormap)NULL) {
       fprintf( stderr, "Error: can't create XColor colormap, exiting.\n");
       exit(1);
    }
  }

  /* Set the colormap and title */
  XtVaSetValues ( Toplevel, XmNcolormap, cmap, 
                            XmNtitle, version, 
                  NULL );


  /* Create the main window */
  Mainform = XtVaCreateWidget(  "Mainform", xmFormWidgetClass, Toplevel,        
                                 XmNresizable,             True,
        			 NULL );

  /* Load up all of the pixmaps. Why here? Because Toplevel may have a
  ** bogus background color.  We can't load a "good" colormap till after
  ** its up.  This is the first widget with a likely allocated bg color. 
  */
  cmap = InitPixmaps( Mainform, cmap);
  if ( cmap == (Colormap)NULL) {
     fprintf( stderr, "Error: can't create Pixmap colormap, exiting.\n");
     exit(1);
  }

  /* Set the icon */
  MakeIconWindow ( Toplevel );

  /**********************/
 
  /* Create menu bar and pulldown menus */
  n = 0;
  XtSetArg(args[n], XmNtopAttachment,   XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNleftAttachment,   XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNrightAttachment,   XmATTACH_FORM); n++;
  Menubar =  XmCreateMenuBar ( Mainform, "menubar", args, n );

  XtVaCreateManagedWidget("button_0", xmCascadeButtonGadgetClass,Menubar, NULL);
  XtVaCreateManagedWidget("button_1", xmCascadeButtonGadgetClass,Menubar, NULL);
  XtVaCreateManagedWidget("button_2", xmCascadeButtonGadgetClass,Menubar, NULL);
  XtVaCreateManagedWidget("button_3", xmCascadeButtonGadgetClass,Menubar, NULL);
  XtVaCreateManagedWidget("button_4", xmCascadeButtonGadgetClass,Menubar, NULL);
  XtVaCreateManagedWidget("button_5", xmCascadeButtonGadgetClass,Menubar, NULL);
  XtVaCreateManagedWidget("button_6", xmCascadeButtonGadgetClass,Menubar, NULL);

  /* Set help menu */
  if ( (w = XtNameToWidget (Menubar, "button_6")) )
    XtVaSetValues( Menubar, XmNmenuHelpWidget, w, NULL);

  /* 
  ** File menu 
  */
  Filemenu = makeMenuPulldown ( Menubar, "Filemenu", 0, file, XtNumber(file) );
  /* Some don't make sense til file loaded*/
  if ( (w = XtNameToWidget (Filemenu, "button_2")) )   /* close */
     XtVaSetValues( w, XmNsensitive, False, NULL);
  if ( (w = XtNameToWidget (Filemenu, "button_3")) )   /* save */
     XtVaSetValues( w, XmNsensitive, False, NULL);

  /*
  ** Edit menu
  */
  Editmenu = makeMenuPulldown ( Menubar, "Editmenu", 1, edit, XtNumber(edit) );
  /* Some don't make sense til file loaded*/
  if ( (w = XtNameToWidget (Editmenu, "button_0")) )   /* copy */
     XtVaSetValues( w, XmNsensitive, False, NULL);
  if ( (w = XtNameToWidget (Editmenu, "button_1")) )   /* cut */
     XtVaSetValues( w, XmNsensitive, False, NULL);

  /* 
  ** Tool menu 
  */
  Toolmenu = makeMenuPulldown ( Menubar, "Toolmenu", 2, tool, XtNumber(tool) );
  /* Some don't make sense til file loaded */
  if ( (w = XtNameToWidget (Toolmenu, "button_1")) )   /* total return */
     XtVaSetValues( w, XmNsensitive, False, NULL);
  if ( (w = XtNameToWidget (Toolmenu, "button_2")) )   /* plot */
     XtVaSetValues( w, XmNsensitive, False, NULL);
  if ( (w = XtNameToWidget (Toolmenu, "button_4")) )   /* portfolio */
     XtVaSetValues( w, XmNsensitive, False, NULL);

  /* 
  ** Option menu 
  */
  Optionmenu = makeMenuPulldown ( Menubar, "Optionmenu", 3, 
                                  option, XtNumber(option) );
  if ( (w = XtNameToWidget (Optionmenu, "button_0")) )   /* zoom */
     XtVaSetValues( w, XmNsensitive, False, NULL);
  if ( (w = XtNameToWidget (Optionmenu, "button_1")) )   /* show points */
     XtVaSetValues( w, XmNsensitive, False, NULL);
  if ( (w = XtNameToWidget (Optionmenu, "button_2")) )   /* show grid */
     XtVaSetValues( w, XmNsensitive, False, NULL);

  /* 
  ** Account menu 
  */
  Accountmenu = makeMenuPulldown ( Menubar, "Accountmenu", 4,
                                   account, XtNumber(account) );
  /* Some don't make sense til file loaded*/
  if ( (w = XtNameToWidget (Accountmenu, "button_0")) )   /* process */
     XtVaSetValues( w, XmNsensitive, False, NULL);
  if ( (w = XtNameToWidget (Accountmenu, "button_1")) )   /* cycle */
     XtVaSetValues( w, XmNsensitive, False, NULL);

  /* 
  ** Log menu 
  */
  makeMenuPulldown ( Menubar, "Viewmenu", 5, log, XtNumber(log) );

  /* 
  ** Help menu 
  */
  makeMenuPulldown ( Menubar, "Helpmenu", 6, help, XtNumber(help) );

  XtManageChild (Menubar);
   
  /* Set "about" as initial selection */
  w = XtNameToWidget ( Toolmenu, "button_0" );
  XtVaSetValues ( w, XmNset, True, NULL);

  /**********************/

  /* Create work area form */
  Workform = XtVaCreateWidget ( "Workform", xmFormWidgetClass, Mainform, 
			        XmNtopAttachment,    XmATTACH_WIDGET,
                                XmNtopWidget,        Menubar,
			        XmNleftAttachment,   XmATTACH_FORM,
			        XmNrightAttachment,  XmATTACH_FORM,
                                XmNbottomAttachment, XmATTACH_FORM,
			        NULL );
  /**********************/

#if XmVERSION > 1
  /* Create pane for left and center areas */
  HPane = XtVaCreateWidget ( "HPane", xmPanedWindowWidgetClass, Workform,
                             XmNorientation,      XmHORIZONTAL,
			     XmNtopAttachment,    XmATTACH_FORM,
                             XmNbottomAttachment, XmATTACH_FORM,
			     XmNleftAttachment,   XmATTACH_FORM,
	               	     NULL ); 
  /* Create pane for text and graph */
  VPane = XtVaCreateWidget ( "VPane", xmPanedWindowWidgetClass, HPane,
                            XmNorientation,     XmVERTICAL,
	               	    NULL ); 
#else
  /* Create pane for text and graph */
  VPane = XtVaCreateWidget ( "VPane", xmPanedWindowWidgetClass, Workform,
		            XmNtopAttachment,    XmATTACH_FORM,
			    XmNleftAttachment,   XmATTACH_FORM,
		            XmNbottomAttachment, XmATTACH_FORM,
	               	    NULL ); 
#endif
  /*********************/

  Translabel = XtVaCreateManagedWidget("Translabel",
                          xmLabelWidgetClass, VPane,
                          NULL);
  {
    Dimension height, margin;
    XtVaGetValues (Translabel, XmNheight, &height,
                               XmNmarginHeight, &margin,
                   NULL);
    XtVaSetValues (Translabel, XmNpaneMinimum, height + 2*margin,
                               XmNpaneMaximum, height + 2*margin,
                   NULL); 
  }
  /*********************/

  /* Scrolled text widget holding transaction history*/
  n = 0;
  XtSetArg(args[n], XmNeditMode,  XmMULTI_LINE_EDIT); n++;
  Transtext = XmCreateScrolledText( VPane, "Transtext", args, n);
  XtAddCallback( Transtext, XmNgainPrimaryCallback,
                 (XtCallbackProc) editSelectionCB, (XtPointer) NULL);
  XtAddCallback( Transtext, XmNmotionVerifyCallback,
                 (XtCallbackProc) editSelectionCB, (XtPointer) NULL);
  XtAddCallback( Transtext, XmNlosePrimaryCallback,
                 (XtCallbackProc) editSelectionCB, (XtPointer) NULL);

  XtManageChild (Transtext);
  XtSetSensitive (Transtext, False);
  /*******************/

  /* Drawing area to graph cost */
  Graphframe = XtVaCreateManagedWidget( "Graphframe", 
					xmFrameWidgetClass, VPane,
					NULL);

  Graph = XtVaCreateWidget ( "Graph", 
                             xmDrawingAreaWidgetClass, Graphframe, 
                             NULL );

  XtAddCallback( Graph, XmNexposeCallback, 
                   (XtCallbackProc) redrawDrawing, (XtPointer) 1);
#if 0
  XtAddCallback( Graph, XmNinputCallback, 
                   (XtCallbackProc) redrawDrawing, (XtPointer) NULL);
#endif

  /* Set gc for Xlib drawing */
  XtVaSetValues ( Graph, XmNuserData, gc, NULL );

  /* tell drawing routines where the drawing area is */
  setDrawingArea ( Graph );

  /* Get fonts for graph titles, etc. */

  small = large = NULL;
  if ( XmFontListInitFontContext ( &fontContext, appdata.graph_font_list ) 
       == True ) {
  
    XmFontListEntry entry;
    XmFontType      fonttype =  XmFONT_IS_FONT;
    char *tag;
    
    while ( (entry = XmFontListNextEntry ( fontContext )) != NULL ) {
      
       tag = XmFontListEntryGetTag ( entry );
       if ( strcmp( tag, "small") == 0 )
         small = (XFontStruct *) XmFontListEntryGetFont ( entry, &fonttype );
       if ( strcmp( tag, "large") == 0 )
         large = (XFontStruct *) XmFontListEntryGetFont ( entry, &fonttype );

       if ( tag != NULL )
         XtFree (tag);

    }

    XmFontListFreeFontContext ( fontContext );
  }

  if ( small == NULL || large == NULL ) {
    fprintf (stderr, "Error: unable to load drawing area fonts. \n");
    fprintf (stderr, "Xinvest will crash on a graph operation! \n");
  }
       
  XtManageChild(Graph);

  /******************/


  /* Create form to keep file action and status forms tidy */
#if XmVERSION > 1
  Midform = XtVaCreateWidget ( "Midform", xmFormWidgetClass, HPane, 
			        NULL );
#else
  Midform = XtVaCreateWidget ( "Midform", xmFormWidgetClass, Workform, 
			       	XmNtopAttachment,    XmATTACH_FORM,
			       	XmNbottomAttachment, XmATTACH_FORM,
			        NULL );
 
  /* Now we have a Midform, attach Pane to it */
  XtVaSetValues ( VPane, XmNrightAttachment,  XmATTACH_WIDGET,
                         XmNrightWidget,      Midform,
                  NULL );
#endif

  /* File area frame */
  Fileframe = XtVaCreateManagedWidget( "Fileframe", 
					xmFrameWidgetClass, Midform,
			       		XmNtopAttachment,   XmATTACH_FORM,
			       		XmNleftAttachment,  XmATTACH_FORM,
			       		XmNrightAttachment, XmATTACH_FORM,
			         	NULL );
  /* title for frame */
  Filecwdlabel = XtVaCreateManagedWidget("File Manager",
                          xmLabelGadgetClass, Fileframe,
                          XmNchildType, XmFRAME_TITLE_CHILD,
                          XmNchildVerticalAlignment, XmALIGNMENT_CENTER,
                          NULL);
  
  /* File area form */
  Fileform = XtVaCreateWidget ( "Fileform", xmFormWidgetClass, Fileframe, 
			        NULL );
  XtAddEventHandler ( Fileform, LeaveWindowMask, False,
                      showDetailEH, (XtPointer) 1);
  /**********************/

  /* Create new file button */
  Newbut =  XtVaCreateManagedWidget( "New",
         xmPushButtonWidgetClass, Fileform,
         XmNlabelType,            XmPIXMAP,
         XmNlabelPixmap,          GetPixmap(PNEW, NORMAL),
         XmNlabelInsensitivePixmap,   GetPixmap(PNEW, INSENS), 
         XmNsensitive,            True,
         XmNtopAttachment,        XmATTACH_FORM,       
         XmNleftAttachment,       XmATTACH_FORM,       
         NULL);
  XtAddCallback ( Newbut, XmNactivateCallback, fileCB, (XtPointer) 0);

  XtAddEventHandler ( Newbut, EnterWindowMask | LeaveWindowMask, False,
                      showDetailEH, (XtPointer) NULL);

  /* Create open file button */
  Openbut =  XtVaCreateManagedWidget( "Open",
         xmPushButtonWidgetClass,  Fileform,
         XmNlabelType,             XmPIXMAP,
         XmNlabelPixmap,           GetPixmap(POPEN, NORMAL),
         XmNlabelInsensitivePixmap,   GetPixmap(POPEN, INSENS), 
         XmNsensitive,             True,
         XmNtop,                   XmATTACH_FORM,       
         XmNleftAttachment,        XmATTACH_WIDGET,       
         XmNleftWidget,            Newbut,       
         NULL);
  XtAddCallback ( Openbut, XmNactivateCallback, fileCB, (XtPointer) 1);

  XtAddEventHandler ( Openbut, EnterWindowMask | LeaveWindowMask, False,
                      showDetailEH, (XtPointer) NULL);

  /* Create save file button */
  Savebut =  XtVaCreateManagedWidget( "Save",
         xmPushButtonWidgetClass, Fileform,
         XmNlabelType,            XmPIXMAP,
         XmNlabelPixmap,          GetPixmap(PSAVE, NORMAL),
         XmNlabelInsensitivePixmap,   GetPixmap(PSAVE, INSENS), 
         XmNsensitive,            False,
         XmNtopAttachment,        XmATTACH_WIDGET,       
         XmNtopWidget,            Openbut,       
         XmNleftAttachment,       XmATTACH_FORM,       
         NULL);
  /* So we can sensitize the menu, when we do the button */
  if ( (w = XtNameToWidget (Filemenu, "button_3")) )   /* save */
     XtVaSetValues( Savebut, XmNuserData, w, NULL);
  XtAddCallback ( Savebut, XmNactivateCallback, fileCB, (XtPointer) 3);

  XtAddEventHandler ( Savebut, EnterWindowMask | LeaveWindowMask, False,
                      showDetailEH, (XtPointer) NULL);


  /* Create cycle account button */
  Cyclebut =  XtVaCreateManagedWidget( "Cyclebut",
         xmPushButtonWidgetClass, Fileform,
         XmNlabelType,            XmPIXMAP,
         XmNlabelPixmap,          GetPixmap(PCYCLE, NORMAL),
         XmNlabelInsensitivePixmap,   GetPixmap(PCYCLE, INSENS), 
         XmNsensitive,            False,
         XmNtopAttachment,        XmATTACH_WIDGET,       
         XmNtopWidget,            Newbut,       
         XmNleftAttachment,       XmATTACH_WIDGET,       
         XmNleftWidget,           Savebut,       
         NULL);

  /* We can sensitize the menu, when we do the button */
  if ( (w = XtNameToWidget (Accountmenu, "button_0")) )   /* cycle */
     XtVaSetValues( Cyclebut, XmNuserData, w, NULL);

  XtAddCallback ( Cyclebut, XmNactivateCallback, accountCB, (XtPointer)0 );

  XtAddEventHandler ( Cyclebut, EnterWindowMask | LeaveWindowMask, False,
                      showDetailEH, (XtPointer) NULL);


  /* Create file list window */
  n = 0;
  XtSetArg(args[n], XmNselectionPolicy,  XmBROWSE_SELECT); n++;
  XtSetArg(args[n], XmNtopAttachment,    XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNleftAttachment,   XmATTACH_WIDGET); n++;
  XtSetArg(args[n], XmNleftWidget,       Openbut); n++;
  XtSetArg(args[n], XmNrightAttachment,  XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNuserData,         NULL); n++;
  Listfile = XmCreateScrolledList(Fileform, "Listfile", args, n);
  XtAddCallback ( Listfile, XmNdefaultActionCallback, fileCB, (XtPointer) 1);
  XtManageChild(Listfile);

  if ( getcwd( dir, BUFSIZ) )
    loadfile( dir );

  /**************************/


  /* Action area form */
  Actframe = XtVaCreateManagedWidget( "Actframe", 
				      xmFrameWidgetClass, Midform,
			              XmNleftAttachment,  XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_FORM,
                                      NULL);

  /* Now we have a Actframe, attach Fileframe to it */
  XtVaSetValues ( Fileframe, XmNbottomAttachment,  XmATTACH_WIDGET,
                             XmNbottomWidget,      Actframe,
                  NULL );

  /* title for frame */
  XtVaCreateManagedWidget("Tool Manager",
                          xmLabelGadgetClass, Actframe,
                          XmNchildType, XmFRAME_TITLE_CHILD,
                          XmNchildVerticalAlignment, XmALIGNMENT_CENTER,
                          NULL);
  
  Actform = XtVaCreateWidget ( "Actform", xmFormWidgetClass, Actframe, 
                                NULL );

  /**********************/
  /* The info label */
  Infoframe = XtVaCreateManagedWidget( "Infoframe", 
				      xmFrameWidgetClass, Actform,
			              XmNtopAttachment,   XmATTACH_SELF,
			              XmNleftAttachment,  XmATTACH_SELF,
                                      NULL);
  XtVaCreateManagedWidget( "Infolabel",
                           xmLabelWidgetClass,      Infoframe,
                           XmNlabelType,            XmPIXMAP,
                           XmNlabelPixmap,          GetPixmap(PINFO, NORMAL),
                           NULL);
  
  /* Make the graph selection area for actform */

  Graphform = XtVaCreateWidget ( "Graphform", xmFormWidgetClass, Actform, 
                               XmNtopAttachment,  XmATTACH_SELF,
                               XmNleftAttachment, XmATTACH_SELF,
                               NULL );

  /* account or portfolio choices */
  Graphpframe = XtVaCreateManagedWidget( "Graphpframe", 
			       xmFrameWidgetClass, Graphform,
                               XmNtopAttachment,  XmATTACH_FORM,
                               XmNleftAttachment, XmATTACH_FORM,
		NULL);

  XtVaCreateManagedWidget("GraphMode",
                          xmLabelGadgetClass, Graphpframe,
                          XmNchildType, XmFRAME_TITLE_CHILD,
                          NULL);

  Graphprow = XmVaCreateSimpleRadioBox ( Graphpframe, "Graphprow", 0,
                                   graph_pselect,
                                   XmVaRADIOBUTTON, NULL, NULL, NULL, NULL,
                                   XmVaRADIOBUTTON, NULL, NULL, NULL, NULL,
                                   NULL);

  if ( (w = XtNameToWidget ( Graphprow, "button_0" )) != NULL)
    XtVaSetValues ( w,
                    XmNlabelType, XmPIXMAP,
                    XmNlabelPixmap, GetPixmap (PACCT, NORMAL),
                    NULL);
  if ( (w = XtNameToWidget ( Graphprow, "button_1" )) != NULL)
    XtVaSetValues ( w,
                    XmNlabelType, XmPIXMAP,
                    XmNlabelPixmap, GetPixmap (PACCTS, NORMAL),
                    NULL);
  XtManageChild( Graphprow );

  /* horizontal choices */
  Graphhframe = XtVaCreateManagedWidget( "Graphhframe", 
		               xmFrameWidgetClass, Graphform,
                               XmNtopAttachment,  XmATTACH_FORM,
                               XmNleftAttachment, XmATTACH_WIDGET,
                               XmNleftWidget, Graphpframe,
		NULL);

  XtVaCreateManagedWidget("GraphType",
                          xmLabelGadgetClass, Graphhframe,
                          XmNchildType, XmFRAME_TITLE_CHILD,
                          NULL);

  Graphhrow = XmVaCreateSimpleRadioBox ( Graphhframe, "Graphhrow", 0,
                                   graph_hselect,
                                   XmVaRADIOBUTTON, NULL, NULL, NULL, NULL,
                                   XmVaRADIOBUTTON, NULL, NULL, NULL, NULL,
                                   NULL);
  if ( (Graphhbar = XtNameToWidget ( Graphhrow, "button_0" )) != NULL)
    XtVaSetValues ( Graphhbar,
                    XmNlabelType, XmPIXMAP,
                    XmNlabelPixmap, GetPixmap (PBAR, NORMAL),
                    XmNlabelInsensitivePixmap, GetPixmap (PBAR, INSENS),
                    NULL);
  if ( (Graphhline = XtNameToWidget ( Graphhrow, "button_1" )) != NULL)
    XtVaSetValues ( Graphhline,
                    XmNlabelType, XmPIXMAP,
                    XmNlabelPixmap, GetPixmap (PLINE, NORMAL),
                    NULL);
  XtManageChild( Graphhrow );
  
  /* vertical choices */
  Graphvframe = XtVaCreateManagedWidget( "Graphvframe", 
			       xmFrameWidgetClass, Graphform,
                               XmNtopAttachment,  XmATTACH_WIDGET,
                               XmNtopWidget,      Graphpframe,
                               XmNleftAttachment, XmATTACH_FORM,
			       NULL);

  XtVaCreateManagedWidget("GraphVar",
                          xmLabelGadgetClass, Graphvframe,
                          XmNchildType, XmFRAME_TITLE_CHILD,
                          NULL);

  Graphvform  = XtVaCreateWidget ( "Graphvform", xmFormWidgetClass, 
                                   Graphvframe, NULL );


{
  Widget row, form, button;
  Pixel  background;
  Pixmap legendoff, legendon;

  /* Make a scrolled window with a checkbox in it holding all variables
  ** we are willing to plot. */
  Graphvscroll = XtVaCreateManagedWidget ( "Graphvscroll", 
                                 xmScrolledWindowWidgetClass, Graphvform,
                                 XmNscrollingPolicy, XmAUTOMATIC,
                                 XmNtopAttachment, XmATTACH_FORM,
                                 XmNleftAttachment, XmATTACH_FORM,
                                 NULL);
  XtAddCallback ( Graphvscroll, 
                  XmNtraverseObscuredCallback, graphTraverse,
                  NULL );

  row = XtVaCreateWidget ( "Graphvrow", xmRowColumnWidgetClass, Graphvscroll,
                                 XmNnumColumns, NUM_PLOTS,
                                 XmNorientation, XmHORIZONTAL,
                                 XmNpacking, XmPACK_COLUMN,
                                 XmNuserData, 0,
                                 NULL );

  legendoff = XCreatePixmap ( XtDisplay(Toplevel),
                              RootWindowOfScreen( XtScreen(Toplevel) ),
                              LEGEND_WIDTH, LEGEND_HEIGHT,
                              DefaultDepthOfScreen( XtScreen(Toplevel) ) );

  /* Use this color for legend background */
  XtVaGetValues ( Graph, XmNbackground, &background, NULL);

  for (i = 0; i < NUM_PLOTS; i++) {
    char name[10];

    sprintf( name, "form_%d", i);
    form  = XtVaCreateWidget ( name, 
                               xmFormWidgetClass, row, 
                               XmNnavigationType, XmNONE,
                               NULL);

    legendon = XCreatePixmap ( XtDisplay(Toplevel),
                               RootWindowOfScreen( XtScreen(Toplevel) ),
                               LEGEND_WIDTH, LEGEND_HEIGHT,
                               DefaultDepthOfScreen( XtScreen(Toplevel) ) );


    sprintf( name, "button_%d", i);
    button = XtVaCreateManagedWidget ( name,
                          xmToggleButtonWidgetClass, form,
                          XmNindicatorType,  XmN_OF_MANY,
                          XmNlabelType, XmPIXMAP,
                          XmNlabelPixmap, legendoff,
                          XmNlabelInsensitivePixmap, legendoff,
                          XmNselectPixmap, legendon,
                          XmNuserData, i,
                          XmNleftAttachment, XmATTACH_FORM,
                          XmNtopAttachment, XmATTACH_FORM,
                          XmNbottomAttachment, XmATTACH_FORM,
             NULL );
    XtAddCallback ( button, XmNvalueChangedCallback,
                    (XtCallbackProc) graph_vselect, (XtPointer) i );

    makeLegend ( button, LEGEND_INIT, background);

    /* Label describing which plot type this toggle is for */
    sprintf( name, "label_%d", i);
    XtVaCreateManagedWidget ( name,
                          xmLabelWidgetClass, form,
                          XmNtraversalOn, False,
                          XmNleftAttachment, XmATTACH_WIDGET,
                          XmNleftWidget, button,
                          XmNtopAttachment, XmATTACH_FORM,
                          XmNbottomAttachment, XmATTACH_FORM,
                          NULL);

    XtManageChild (form);
  }

  if ( (GraphconstToggle = XtNameToWidget (row, "form_7")) )   /* constant */
     XtSetSensitive ( GraphconstToggle, False);

  XtManageChild (row);

}
  /* Make a form so that the all widgets end up the same height */
  Graphcform  = XtVaCreateWidget ( "Graphcform", 
                                  xmFormWidgetClass, Graphform, 
                                  XmNnavigationType, XmNONE,
                                  XmNsensitive,       False,
                                  XmNtopAttachment,  XmATTACH_WIDGET,
                                  XmNtopWidget,      Graphvframe,
                                  XmNleftAttachment, XmATTACH_FORM,
                NULL );

  Graphvlabel = XtVaCreateManagedWidget( "Graphvlabel", 
			        xmLabelWidgetClass, Graphcform,
                                XmNbottomAttachment, XmATTACH_FORM,
                                XmNtopAttachment,    XmATTACH_FORM,
                                XmNleftAttachment,   XmATTACH_FORM,
               NULL );

  Graphvtext1 = XtVaCreateManagedWidget ("Graphvstart", xmTextFieldWidgetClass, 
                                Graphcform,
                                XmNeditable,         True,
                                XmNtopAttachment,    XmATTACH_FORM,
                                XmNbottomAttachment, XmATTACH_FORM,
                                XmNleftAttachment,   XmATTACH_WIDGET,
                                XmNleftWidget,       Graphvlabel,
                NULL );
  XtAddCallback ( Graphvtext1, XmNactivateCallback, 
                  (XtCallbackProc) const_change, (XtPointer) 0);

  Graphvlabel = XtVaCreateManagedWidget( "Graphvlabel", 
			        xmLabelWidgetClass, Graphcform,
			        XtVaTypedArg, XmNlabelString, XmRString,
			        "@", sizeof("@"),  
                                XmNbottomAttachment, XmATTACH_FORM,
                                XmNtopAttachment,    XmATTACH_FORM,
                                XmNleftAttachment,   XmATTACH_WIDGET,
                                XmNleftWidget,       Graphvtext1,
               NULL );

  Graphvtext2 = XtVaCreateManagedWidget ("Graphvrate", xmTextFieldWidgetClass, 
                                Graphcform,
                                XmNeditable,         True,
                                XmNtopAttachment,    XmATTACH_FORM,
                                XmNbottomAttachment, XmATTACH_FORM,
                                XmNleftAttachment,   XmATTACH_WIDGET,
                                XmNleftWidget,       Graphvlabel,
               NULL );
  XtAddCallback ( Graphvtext2, XmNactivateCallback, 
                    (XtCallbackProc) const_change, (XtPointer) 0);

  XtManageChild( Graphcform );

  /* Units for the scale, weeks or transactions */
  Graphslabel = XtVaCreateManagedWidget( "Graphslabel", 
	       xmLabelWidgetClass, Graphform,
               XmNtopAttachment,     XmATTACH_OPPOSITE_WIDGET,
               XmNtopWidget,         Graphvframe,
               XmNrightAttachment,   XmATTACH_FORM,
               XmNleftAttachment,    XmATTACH_WIDGET,
               XmNleftWidget,        Graphvframe,
               NULL);

  /* The horizontal control can change the labelString */
  XtVaSetValues ( Graphhrow, XmNuserData, Graphslabel, NULL);

  Graphscale = XtVaCreateManagedWidget ("Graphscale", 
               xmScaleWidgetClass,   Graphform, 
               XmNtopAttachment,     XmATTACH_WIDGET,
               XmNtopWidget,         Graphslabel,
               XmNleftAttachment,    XmATTACH_WIDGET,
               XmNleftWidget,        Graphvframe,
               XmNrightAttachment,   XmATTACH_FORM,
               XmNbottomAttachment,  XmATTACH_OPPOSITE_WIDGET,
               XmNbottomWidget,      Graphvframe,
               NULL);
  XtAddCallback ( Graphscale, XmNvalueChangedCallback, 
                    (XtCallbackProc) graph_slide, (XtPointer) 0);
  XtSetSensitive ( Graphscale, False );


  XtManageChild( Graphvform );


  /* The total return action area */
  Gainform = XtVaCreateWidget ( "Gainform", xmFormWidgetClass, Actform, 
                               XmNtopAttachment,  XmATTACH_SELF,
                               XmNleftAttachment, XmATTACH_SELF,
            NULL );
 
  Gaincalframe = XtVaCreateManagedWidget( "Gaincalframe", 
                                          xmFrameWidgetClass, Gainform, 
                                          XmNtopAttachment, XmATTACH_FORM,
                                          XmNleftAttachment, XmATTACH_FORM,
                                          NULL );
 
  Gaincalrow = XtVaCreateWidget ( "Gaincalrow", xmRowColumnWidgetClass, 
                                  Gaincalframe,
                                  NULL );

    
  /* create strings containing month names */
  for (month = 0; month < XtNumber (months); month++) {
    strs[month] = XmStringCreateLocalized ( months[month].name );
  }
    

  Gaindaterow = XtVaCreateManagedWidget( "Gaindaterow", xmRowColumnWidgetClass,
                                Gaincalrow,
                                NULL );

  Gainmonthoptmenu = XmVaCreateSimpleOptionMenu( Gaindaterow, 
                     "Gainmonthoptmenu", 
                     NULL, (KeySym) NULL, 0, 
                     (XtCallbackProc) set_month,
                     XmVaPUSHBUTTON, strs[0], 'J', NULL, NULL,
                     XmVaPUSHBUTTON, strs[1], 'F', NULL, NULL,
                     XmVaPUSHBUTTON, strs[2], 'M', NULL, NULL,
                     XmVaPUSHBUTTON, strs[3], 'p', NULL, NULL,
                     XmVaPUSHBUTTON, strs[4], 'a', NULL, NULL,
                     XmVaPUSHBUTTON, strs[5], 'u', NULL, NULL,
                     XmVaPUSHBUTTON, strs[6], 'l', NULL, NULL,
                     XmVaPUSHBUTTON, strs[7], 'g', NULL, NULL,
                     XmVaPUSHBUTTON, strs[8], 'S', NULL, NULL,
                     XmVaPUSHBUTTON, strs[9], 'O', NULL, NULL,
                     XmVaPUSHBUTTON, strs[10], 'N', NULL, NULL,
                     XmVaPUSHBUTTON, strs[11], 'D', NULL, NULL,
                     NULL);

  Gainyeartext = XtVaCreateManagedWidget ("Gainyeartext", 
                                   xmTextFieldWidgetClass, Gaindaterow,
                                   XmNeditable,    False,
                                   XmNcursorPositionVisible, False, 
                                   XmNtraversalOn, False,
                                   NULL );


  GainyearUparrow = XtVaCreateManagedWidget ("GainyearUparrow",
                                xmArrowButtonWidgetClass, Gaindaterow,
                                XmNarrowDirection,   XmARROW_UP,
                                NULL );
  XtAddCallback (  GainyearUparrow, XmNactivateCallback, 
                   (XtCallbackProc) year_updown, (XtPointer) 1);

  GainyearDnarrow = XtVaCreateManagedWidget ("GainyearDnarrow",
                                xmArrowButtonWidgetClass, Gaindaterow,
                                XmNarrowDirection,   XmARROW_DOWN,
                                NULL );
  XtAddCallback (  GainyearDnarrow, XmNactivateCallback, 
                   (XtCallbackProc) year_updown, (XtPointer) -1);


  {
    /* This creates an initial calendar so that the calendar size can be
    ** taken into the calculations before freezing the action area size.
    */
    time_t t = time ( (time_t *)0 );
    struct tm *today = localtime (&t);

    displayCal (  today->tm_mon +1, today->tm_mday, 1900 + today->tm_year);
  }

  for (month = 0; month < XtNumber (months); month++)
     XmStringFree ( strs[month] );

  XtManageChild ( Gainmonthoptmenu );


  /* Create Gain NAV value calculator */
  Gainvalframe = XtVaCreateManagedWidget( "Gainvalframe", 
                                  xmFrameWidgetClass, Gainform, 
                                  XmNtopAttachment, XmATTACH_FORM,
                                  XmNleftAttachment, XmATTACH_WIDGET,
                                  XmNleftWidget, Gaincalframe,
                                  NULL );
 
  Gainvalrow = XtVaCreateWidget ( "Gainvalrow", xmRowColumnWidgetClass,
                                   Gainvalframe, 
                                   XmNentryAlignment, XmALIGNMENT_CENTER,
                                   NULL );

  Gaincalctext = XtVaCreateManagedWidget ("Gaincalctext", 
                                   xmTextFieldWidgetClass, Gainvalrow,
                                   XmNeditable, False,
                                   XmNcursorPositionVisible, False, 
                                   XmNtraversalOn, False,
                                   NULL );

  Gaincalcrow = XtVaCreateWidget ( "Gaincalcrow", xmRowColumnWidgetClass,
                                   Gainvalrow, 
                                   XmNnavigationType, XmNONE,
                                   NULL );


  /* loop though digit values */
  for ( i = 0; i < XtNumber (digits) -1; i++ ) {
    w =  XtVaCreateManagedWidget ( digits[i],
                                   xmPushButtonWidgetClass,
                                   Gaincalcrow, NULL );
    XtAddCallback ( w, XmNactivateCallback, 
                   (XtCallbackProc) calc_push, (XtPointer) i);  
  }

  /* create compute button */
  w = XtVaCreateManagedWidget ( digits[i], 
                                xmPushButtonWidgetClass, Gainvalrow, 
                                XmNnavigationType, XmNONE,
                                NULL );
  XtAddCallback ( w, XmNactivateCallback, 
                  (XtCallbackProc) calc_push, (XtPointer) i);  

  XtManageChild ( Gaincalcrow );
  XtManageChild ( Gainvalrow );

  /* Total return buttons */

  Gainbutframe = XtVaCreateManagedWidget( "Gainbutframe", 
                  xmFrameWidgetClass, Gainform, 
                  XmNtopAttachment,    XmATTACH_WIDGET,
                  XmNtopWidget,        Gainvalframe,
                  XmNleftAttachment,   XmATTACH_WIDGET,
                  XmNleftWidget,       Gaincalframe,
                  XmNrightAttachment,  XmATTACH_OPPOSITE_WIDGET,
                  XmNrightWidget,      Gainvalframe,
                  XmNbottomAttachment, XmATTACH_FORM,
                  NULL); 

  /* Point to pixmaps we want */
  gainbar[0].sens = GetPixmap( PLAST, NORMAL);
  gainbar[1].sens = GetPixmap( PACCT, NORMAL);
  gainbar[1].sel = GetPixmap( PACCTS, NORMAL);

  Gainbutform = makeButtonbar ( Gainbutframe, "Gainbutform", XmHORIZONTAL, 
                                gainbar, XtNumber(gainbar) );
  XtManageChild ( Gainbutform );
 
  /********************/
    

  /* Make the portfolio action area */
  Portform = XtVaCreateWidget ( "Portform", xmFormWidgetClass, Actform, 
                               XmNtopAttachment,  XmATTACH_SELF,
                               XmNleftAttachment, XmATTACH_SELF,
                               NULL );

  /* Row column for the two button areas */
  Portselrow = XtVaCreateWidget ( "Portselrow", xmRowColumnWidgetClass, 
                                  Portform,
                                  XmNtopAttachment,  XmATTACH_FORM,
                                  XmNleftAttachment, XmATTACH_FORM,
                                  XmNrightAttachment,XmATTACH_FORM,
                                  NULL );

  /* Each button area is a frame, rowcolumn and two buttons */
  /* Frame and form for single account buttons */
  Portframe = XtVaCreateManagedWidget( "Portframe", xmFrameWidgetClass,
                                       Portselrow,
                                       NULL);
 
  XtVaCreateManagedWidget("PortAccount",
                          xmLabelGadgetClass, Portframe,
                          XmNchildType, XmFRAME_TITLE_CHILD,
                          NULL);

  Portbutform = XtVaCreateWidget ( "Portbutform", xmFormWidgetClass, 
                                   Portframe,
                                   NULL );

  /* + or - for current account */
  Portpbut =  XtVaCreateManagedWidget ( "Add", 
                       xmPushButtonWidgetClass, Portbutform, 
                       XmNlabelType,            XmPIXMAP,
                       XmNlabelPixmap,          GetPixmap(PCHECK, NORMAL),
                       XmNlabelInsensitivePixmap,  GetPixmap(PCHECK, INSENS),
                       XmNsensitive,            True,
                       XmNtopAttachment,        XmATTACH_FORM,
                       XmNleftAttachment,       XmATTACH_POSITION,
                       XmNrightAttachment,      XmATTACH_POSITION,
                       NULL );
  XtAddCallback ( Portpbut, XmNactivateCallback, 
                 (XtCallbackProc) portAddSubList, (XtPointer) 0);  

  Portmbut =  XtVaCreateManagedWidget ( "Remove", 
                       xmPushButtonWidgetClass, Portbutform, 
                       XmNlabelType,            XmPIXMAP,
                       XmNlabelPixmap,          GetPixmap(PX, NORMAL),
                       XmNlabelInsensitivePixmap,  GetPixmap(PX, INSENS),
                       XmNsensitive,            False,
                       XmNtopAttachment,        XmATTACH_FORM,
                       XmNleftAttachment,       XmATTACH_POSITION,
                       XmNrightAttachment,      XmATTACH_POSITION,
                       NULL );
  XtAddCallback ( Portmbut, XmNactivateCallback, 
                 (XtCallbackProc) portAddSubList, (XtPointer) 1);  

  XtManageChild ( Portbutform );


  /* Frame and form for all accounts buttons */
  Portframe = XtVaCreateManagedWidget( "Portframe", xmFrameWidgetClass,
                                       Portselrow,
                                       NULL);
 
  XtVaCreateManagedWidget("PortAccounts",
                          xmLabelGadgetClass, Portframe,
                          XmNchildType, XmFRAME_TITLE_CHILD,
                          NULL);

  Portbutform = XtVaCreateWidget ( "Portbutform", xmFormWidgetClass, 
                                   Portframe,
                                   NULL );

  Portabut = XtVaCreateManagedWidget ( "AddAll", 
                       xmPushButtonWidgetClass, Portbutform, 
                       XmNsensitive,            True,
                       XmNlabelType,            XmPIXMAP,
                       XmNlabelPixmap,          GetPixmap(PCHECK, NORMAL),
                       XmNlabelInsensitivePixmap,  GetPixmap(PCHECK, INSENS),
                       XmNtopAttachment,        XmATTACH_FORM,
                       XmNleftAttachment,       XmATTACH_POSITION,
                       XmNrightAttachment,      XmATTACH_POSITION,
                       NULL );
  XtAddCallback ( Portabut, XmNactivateCallback, 
                 (XtCallbackProc) portAddSubList, (XtPointer) 2);  

  Portcbut =  XtVaCreateManagedWidget ( "Clear", 
                       xmPushButtonWidgetClass, Portbutform, 
                       XmNsensitive,            True,
                       XmNlabelType,            XmPIXMAP,
                       XmNlabelPixmap,          GetPixmap(PX, NORMAL),
                       XmNlabelInsensitivePixmap,  GetPixmap(PX, INSENS),
                       XmNtopAttachment,        XmATTACH_FORM,
                       XmNleftAttachment,       XmATTACH_POSITION,
                       XmNrightAttachment,      XmATTACH_POSITION,
                       NULL );
  XtAddCallback ( Portcbut, XmNactivateCallback, 
                 (XtCallbackProc) portAddSubList, (XtPointer) 3);  

  XtManageChild ( Portbutform );
  XtManageChild ( Portselrow );


  /* Now create the list areas */
  Portscrform = XtVaCreateWidget ( "Portscrform", xmFormWidgetClass, Portform, 
                                 XmNtopAttachment,    XmATTACH_WIDGET,
                                 XmNtopWidget,        Portselrow,
                                 XmNleftAttachment,   XmATTACH_FORM,
                                 XmNrightAttachment,  XmATTACH_FORM,
                                 XmNbottomAttachment, XmATTACH_FORM,
                                 NULL );

  Portpercentlabel = XtVaCreateManagedWidget( "Portpercentlabel", 
			        xmLabelWidgetClass, Portscrform,
                                XmNtopAttachment,    XmATTACH_FORM,
                                XmNleftAttachment,   XmATTACH_FORM,
                                XmNrightAttachment,  XmATTACH_FORM,
                                NULL );
  
  /* Create percentage list window */
  n = 0;
  XtSetArg(args[n], XmNselectionPolicy,  XmSINGLE_SELECT); n++;
  XtSetArg(args[n], XmNtopAttachment,    XmATTACH_WIDGET); n++;
  XtSetArg(args[n], XmNtopWidget,        Portpercentlabel); n++;
  XtSetArg(args[n], XmNleftAttachment,   XmATTACH_FORM); n++;
  XtSetArg(args[n], XmNrightAttachment,  XmATTACH_FORM); n++;
  Portpercentlist = XmCreateScrolledList ( Portscrform, "Portpercentlist", 
                                           args, n);

  XtAddCallback ( Portpercentlist, XmNdefaultActionCallback, 
                  (XtCallbackProc) portTraverse, NULL);

  { /* Set list width, because it will not be resizable later */
    XmFontList      fontlist;
    XmFontContext   context;
    XmFontListEntry entry;
    XmFontType fonttype = XmFONT_IS_FONT;
    XFontStruct *font = NULL;

    XtVaGetValues ( Portpercentlist, XmNfontList, &fontlist, NULL);

    if (fontlist) {
      if ( XmFontListInitFontContext ( &context, fontlist) ) {
        entry = XmFontListNextEntry ( context );
        if (entry)
           font = (XFontStruct *) XmFontListEntryGetFont ( entry, &fonttype );
        XmFontListFreeFontContext ( context );
      }
    }

    if (font)
      XtVaSetValues ( Portpercentlist,
                      XmNwidth, 35*font->max_bounds.width,
                      NULL);
  }

  XtManageChild ( Portpercentlist );

  /* Option menu for moving around the percentage tree */
  for (i=0; i < XtNumber(portoptmenu); i++)
    Temp[i] = XmStringCreateLocalized( portoptmenu[i] );

  Portoptmenu = XmVaCreateSimpleOptionMenu( Portscrform, "Portoptmenu", 
                     Temp[0], (KeySym)'P' , 0, (XtCallbackProc) portTraverse,
                     XmVaPUSHBUTTON, Temp[1], 'R', NULL, NULL,
                     XmVaPUSHBUTTON, Temp[2], 'F', NULL, NULL,
                     XmVaPUSHBUTTON, Temp[3], 'A', NULL, NULL,
                     XmNtopAttachment,        XmATTACH_WIDGET,
                     XmNtopWidget,            Portpercentlist,
                     XmNleftAttachment,       XmATTACH_FORM,
                     NULL);
  XtManageChild ( Portoptmenu );
  XtSetSensitive ( Portoptmenu, False );

  for (i=0; i < XtNumber(portoptmenu); i++)
    XmStringFree ( Temp[i] );


  Portvaluelabel = XtVaCreateManagedWidget( "Portvaluelabel", 
			        xmLabelWidgetClass, Portscrform,
			        XtVaTypedArg, XmNlabelString, XmRString,
			          "Total Portfolio Value: $0", 
                                  sizeof( "Total Portfolio Value: $0" ),  
                                XmNtopAttachment,    XmATTACH_WIDGET,
                                XmNtopWidget,        Portoptmenu,
                                XmNleftAttachment,   XmATTACH_FORM,
                                XmNrightAttachment,  XmATTACH_FORM,
                                XmNbottomAttachment, XmATTACH_FORM,
                                NULL );
  XtManageChild ( Portscrform );
  
  /********************/
    
  /* Make the calculator action area */
  Calcform = XtVaCreateWidget ( "Calcform", xmFormWidgetClass, Actform,
                                 XmNtopAttachment,        XmATTACH_SELF,
                                 XmNleftAttachment,       XmATTACH_SELF,
                                 NULL );

  for (i=0; i < XtNumber(calcfuncs); i++)
    Temp[i] = XmStringCreateLocalized( calcfuncs[i] );

  Calcoptmenu = XmVaCreateSimpleOptionMenu( Calcform, "Calcoptmenu", 
                     Temp[0], (KeySym)'u' , 0, (XtCallbackProc) calcFuncSel,
                     XmVaPUSHBUTTON, Temp[1], 'F', NULL, NULL,
                     XmVaPUSHBUTTON, Temp[2], 'u', NULL, NULL,
                     XmVaPUSHBUTTON, Temp[3], 'P', NULL, NULL,
                     XmVaPUSHBUTTON, Temp[4], 'e', NULL, NULL,
                     XmVaPUSHBUTTON, Temp[5], 'R', NULL, NULL,
                     XmNtopAttachment,    XmATTACH_FORM,
                     XmNleftAttachment,   XmATTACH_FORM,
                     NULL);
  XtManageChild ( Calcoptmenu );

  for (i=0; i < XtNumber(calcfuncs); i++)
    XmStringFree ( Temp[i] );

  Calcframe = XtVaCreateManagedWidget( "Calcframe", xmFrameWidgetClass,
                                Calcform,
                                XmNtopAttachment,    XmATTACH_WIDGET,
                                XmNtopWidget,        Calcoptmenu,
                                XmNleftAttachment,   XmATTACH_FORM,
                                NULL);

  Calclabel = XtVaCreateManagedWidget( "Calclabel", 
			        xmLabelWidgetClass,  Calcframe,
                                XmNlabelType,        XmPIXMAP,
                                XmNlabelPixmap,      GetPixmap(PFV, NORMAL),
                                NULL );
  

  /* Option menu for selecting the function  */
  Calcrow = XtVaCreateWidget ( "Calcrow", xmRowColumnWidgetClass, Calcform,
                               XmNnavigationType, XmNONE,
                               XmNtopAttachment,  XmATTACH_WIDGET,
                               XmNtopWidget,      Calcframe,
                               XmNleftAttachment, XmATTACH_FORM,
                               NULL );

  /* There are only ever four parameters, we'll switch labels as we need to */
  for (i=0; i < 4; i++) {
    Calclabels[i] = XtVaCreateManagedWidget ( calcparams[i],
                              xmLabelGadgetClass, Calcrow,
                              NULL );
    if ( i < 3 ) {
      /* Text widgets for number entry */
      Calctext[i] = XtVaCreateManagedWidget ( calcparams[i], 
                                xmTextFieldWidgetClass, Calcrow,
                                XmNnavigationType, XmNONE,
                                NULL );
      XtAddCallback ( Calctext[i], XmNactivateCallback, 
                      (XtCallbackProc) calcTextSel, (XtPointer) i ); 
    } else {
      /* Option menu for selecting compounding */
      for (i=0; i < XtNumber(calccompnd); i++)
        Temp[i] = XmStringCreateLocalized( calccompnd[i] );

      Calccmpdmenu = XmVaCreateSimpleOptionMenu( Calcrow, "Calccmpdmenu", 
                         Temp[0], (KeySym)0 , 2, (XtCallbackProc) calcCompSel,
                         XmVaPUSHBUTTON, Temp[1], 'D', NULL, NULL,
                         XmVaPUSHBUTTON, Temp[2], 'W', NULL, NULL,
                         XmVaPUSHBUTTON, Temp[3], 'M', NULL, NULL,
                         XmVaPUSHBUTTON, Temp[4], 'Q', NULL, NULL,
                         XmVaPUSHBUTTON, Temp[5], 'B', NULL, NULL,
                         XmVaPUSHBUTTON, Temp[6], 'A', NULL, NULL,
                         XmVaPUSHBUTTON, Temp[7], '1', NULL, NULL,
                         XmVaPUSHBUTTON, Temp[8], '3', NULL, NULL,
                         XmNbottomAttachment,     XmATTACH_OPPOSITE_WIDGET,
                         XmNbottomWidget,         Calcrow,
                         XmNleftAttachment,       XmATTACH_WIDGET,
                         XmNleftWidget,           Calcrow,
                      NULL);
      XtManageChild ( Calccmpdmenu );

      for (i=0; i < XtNumber(calccompnd); i++)
        XmStringFree ( Temp[i] );
    }
  }
  XtManageChild ( Calcrow );

  /* Get a default of the first function. Side effect is setting labels, text
  ** and result string */
  calcFuncSel ( NULL, 0, NULL ); 

  /********************/

  /* Button area form */
  /* Point to pixmaps we want */
  toolbar[0].sens = GetPixmap( PABOUT, NORMAL);
  toolbar[1].sens = GetPixmap( PRETURN, NORMAL);
  toolbar[1].insens = GetPixmap( PRETURN, INSENS);
  toolbar[2].sens = GetPixmap( PGRAPH, NORMAL);
  toolbar[2].insens = GetPixmap( PGRAPH, INSENS);
  toolbar[3].sens = GetPixmap( PFUNCTION, NORMAL);
  toolbar[4].sens = GetPixmap( PPORTFOLIO, NORMAL);
  toolbar[4].insens = GetPixmap( PPORTFOLIO, INSENS);
  toolbar[5].sens = GetPixmap( PSTOP, NORMAL);

  toolbar[1].userdata = (XtPointer) XtNameToWidget (Toolmenu, "button_1");
  toolbar[2].userdata = (XtPointer) XtNameToWidget (Toolmenu, "button_2");
  toolbar[4].userdata = (XtPointer) XtNameToWidget (Toolmenu, "button_4");

  /* Make the button bar */
  Butform = makeButtonbar ( Workform, "Butform", XmVERTICAL, 
                            toolbar, XtNumber(toolbar) );

  /* Add some constraints */
  XtVaSetValues ( Butform,
                  XmNtopAttachment,    XmATTACH_FORM,
                  XmNrightAttachment,  XmATTACH_FORM,
                  XmNbottomAttachment, XmATTACH_FORM,
                  NULL); 

  /* Temporary setting of globals.  Should be replaced by macro or access 
     routine */
  Gainbut = XtNameToWidget (Butform, "Gain");
  Graphbut = XtNameToWidget (Butform, "Graphbut");
  Portbut = XtNameToWidget (Butform, "Portfolio");

#if XmVERSION > 1
  /* Now we have a Butform, attach Midform to it */
  XtVaSetValues ( HPane, XmNrightAttachment,  XmATTACH_WIDGET,
                         XmNrightWidget,      Butform,
                  NULL );
#else
  /* Now we have a Butform, attach Midform to it */
  XtVaSetValues ( Midform, XmNrightAttachment,  XmATTACH_WIDGET,
                           XmNrightWidget,      Butform,
                  NULL );
#endif
  /**********************/


  /* Manage children and off we go */
  XtManageChild( Butform ); 
  XtManageChild( Fileform ); 

  XtVaSetValues ( Graphform, XmNmappedWhenManaged, False, NULL );
  XtManageChild( Graphform );

  XtManageChild ( Gaincalrow );
  XtVaSetValues ( Gainform, XmNmappedWhenManaged, False, NULL );
  XtManageChild ( Gainform );

  XtVaSetValues ( Portform, XmNmappedWhenManaged, False, NULL );
  XtManageChild( Portform );

  XtVaSetValues ( Calcform, XmNmappedWhenManaged, False, NULL );
  XtManageChild( Calcform );

  XtManageChild( Actform ); 
  XtManageChild( Midform );
  XtManageChild( VPane );
#if XmVERSION > 1
  XtManageChild( HPane );
#endif
  XtManageChild( Workform );
  XtManageChild( Mainform );

  XtRealizeWidget ( Toplevel );

  /* Stop midform from resizing */
  XtVaSetValues ( Midform, XmNresizable, False, NULL );
  /* Stop calculator, calendar from resizing */
  XtVaSetValues ( Gaincalrow, XmNresizeHeight, False, 
                              XmNresizeWidth, False, 
                  NULL );

  /* Set Margins of children so they are centered in Actform */
  /* Toplevel must be realized for this to work */
  CenterWidget ( Actform, Infoframe );
  CenterWidget ( Actform, Graphform );
  CenterWidget ( Actform, Gainform );
  CenterWidget ( Actform, Portform );
  CenterWidget ( Actform, Calcform );

  if (appdata.restore)
    restoreState (appdata.restore);

  /* Communication with Xquote */
  remoteInit (Toplevel);

  /* 
  ** If any options left, they must be data files beginning with -f. 
  ** There may be more than one -f option and each can have zero or more
  ** filenames [-f filename(s) -f filename(s) ...].
  */
  if (argc > 1)
    processFileOption ( argc, argv );

  XtAppMainLoop ( app );
}


/* ARGSUSED */
void toolCB ( Widget w, XtPointer client_data, XtPointer call_data)
{
  int which = (int) client_data;

  XtSetMappedWhenManaged ( Gainform,  False );
  XtSetMappedWhenManaged ( Graphform, False );
  XtSetMappedWhenManaged ( Portform,  False );
  XtSetMappedWhenManaged ( Calcform,  False );
  XtSetMappedWhenManaged ( Infoframe,  False );


  switch ( which ) {

  case 1:   /* Total Return */
            setLastGraph (RETURN);
            XtSetMappedWhenManaged ( Gainform, True );
            break;
  case 2:   /* Graph */
            setLastGraph(PLOT);
            XtSetMappedWhenManaged ( Graphform, True );
            break;
  case 3:   /* Calculate */
            setLastGraph(FUNCCALC);
            XtSetMappedWhenManaged ( Calcform, True );
            break;
  case 4:   /* Portfolio */
            setLastGraph(PORTFOLIO);
            XtSetMappedWhenManaged ( Portform, True );
            break;

  default:   /* Label */
            setLastGraph(NOTHING);
            XtSetMappedWhenManaged ( Infoframe, True );
  } 

  /* make new menu the default. */
  {
    extern Widget Toolmenu;
    WidgetList buttons;
    int numbuttons, i;

    XtVaGetValues ( Toolmenu, 
                    XmNchildren, &buttons, 
                    XmNnumChildren, &numbuttons, 
                    NULL );

    /* Don't touch the option buttons. This works because there is a separator 
    ** in the menu before the option buttons. */
    for (i = 0; i < numbuttons; i++)
      XmToggleButtonSetState ( buttons[i], (which == i)?True:False, False);

    /* If plot is on, turn on plot options */
    graphSense( which==2 );
     
  }

  /* sigh */
  drawDrawingArea();
}

/* Action proc for pushing a digit key */
/* ARGSUSED */
static void PushCalcButton (Widget w, XEvent *event, 
                            String *params, Cardinal *num_params)
{
  Boolean mapped;
  int key;
  Widget button = NULL;

  char digit_path[] = "Gainvalframe.Gainvalrow.Gaincalcrow.";
  char comp_path[]  = "Gainvalframe.Gainvalrow.";
  char path[40];

  /* Is calculator visible */
  XtVaGetValues ( Gainform, XmNmappedWhenManaged, &mapped, NULL );
  if ( mapped == True ) {

    for (key=0; key < XtNumber(digits); key++) 

      /* Do we have a key we care about */
      if ( strcasecmp( digits[key], *params ) == 0 ) {

        /* Find the widget for the equivalent pushbutton */
        if (key < (XtNumber(digits) -1) )
          strcpy (path, digit_path);
        else
          strcpy (path, comp_path);
        strcat (path, digits[key]);

        /* Flash the button as a queue to user */
        button = XtNameToWidget ( Gainform, path);

        if (button) 
          XtCallActionProc (button, "ArmAndActivate", event, NULL, 0);
      }
  }
}
